/*	$NetBSD: linux_syscall.c,v 1.3 2024/02/07 04:20:26 msaitoh Exp $	*/

/*
 * Copyright (c) 2021 Ryo Shimizu
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/cdefs.h>
__KERNEL_RCSID(0, "$NetBSD: linux_syscall.c,v 1.3 2024/02/07 04:20:26 msaitoh Exp $");

#if defined(_KERNEL_OPT)
#include "opt_compat_linux.h"
#endif

#include <compat/linux/linux_syscall.h>
#include <compat/linux/common/linux_errno.h>

#define EMULNAME(x)		__CONCAT(linux_,x)
#define EMULNAMEU(x)		__CONCAT(LINUX_,x)


/*
 * syscall.c behavior options
 */

/*
 * linux/arm64 always uses indirect syscall, and ignores code of 'svc #<code>'.
 * The system call number is specified by the x8 register.
 */
#define SYSCALL_CODE_REG	8
#define SYSCALL_NO_INDIRECT

/* used when __HAVE_MINIMAL_EMUL is not defined */
#define SYSCALL_EMUL_ERRNO(x)	native_to_linux_errno[x]

/* don't update x1 register with rval[1] */
#define SYSCALL_NO_RVAL1

#include "syscall.c"
