/*	$NetBSD: bootinfo.h,v 1.1 2003/04/26 18:39:38 fvdl Exp $	*/

#ifndef _AMD64_BOOTINFO_H
#define _AMD64_BOOTINFO_H

#include <x86/bootinfo.h>

#define VAR32_SIZE	4096

#endif	/* _AMD64_BOOTINFO_H_ */
