/*	$NetBSD: bus_defs.h,v 1.1 2011/07/01 17:09:59 dyoung Exp $	*/

#ifndef _IBMNWS_BUS_DEFS_H_
#define _IBMNWS_BUS_DEFS_H_

#include <powerpc/prep_bus_defs.h>
#include <powerpc/bus_defs.h>

#endif /* _IBMNWS_BUS_DEFS_H_ */
