/*	
 * $NetBSD: dev_net.c,v 1.11 2016/08/04 16:22:40 scole Exp $
 */

/*-
 * Copyright (c) 1997 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by Gordon W. Ross.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/cdefs.h>
/* __FBSDID("$FreeBSD: src/sys/boot/common/dev_net.c,v 1.15 2004/07/08 22:35:33 brian Exp $"); */

/*-
 * This module implements a "raw device" interface suitable for
 * use by the stand-alone I/O library NFS code.  This interface
 * does not support any "block" access, and exists only for the
 * purpose of initializing the network interface, getting boot
 * parameters, and performing the NFS mount.
 *
 * At open time, this does:
 *
 * find interface      - netif_open()
 * RARP for IP address - rarp_getipaddress()
 * RPC/bootparams      - callrpc(d, RPC_BOOTPARAMS, ...)
 * RPC/mountd          - nfs_mount(sock, ip, path)
 *
 * the root file handle from mountd is saved in a global
 * for use by the NFS open code (NFS/lookup).
 */

#include <sys/param.h>

#include <lib/libsa/stand.h>
#include <lib/libsa/net.h>
#include <lib/libsa/bootparam.h>
#include <lib/libsa/loadfile.h>
#include <lib/libsa/netif.h>
#include <lib/libsa/nfs.h>
#include <lib/libsa/bootp.h>
#include <lib/libkern/libkern.h>

#include "dev_net.h"
#include "bootstrap.h"

int debug = 0;

static int netdev_sock = -1;
static int netdev_opens;

static int net_getparams(int sock);

/*
 * Called by devopen after it sets f->f_dev to our devsw entry.
 * This opens the low-level device and sets f->f_devdata.
 * This is declared with variable arguments...
 */
int
net_open(struct open_file *f, ...)
{
    va_list args;
    char *devname;		/* Device part of file name (or NULL). */
    int error = 0;

    va_start(args, f);
    devname = va_arg(args, char*);
    va_end(args);

    /* On first open, do netif open, mount, etc. */
    if (netdev_opens == 0) {
	/* Find network interface. */
	if (netdev_sock < 0) {
	    netdev_sock = netif_open(devname);
	    if (netdev_sock < 0) {
		printf("net_open: netif_open() failed\n");
		return (ENXIO);
	    }
	    if (debug)
		printf("net_open: netif_open() succeeded\n");
	}
	if (rootip.s_addr == 0) {
	    /* Get root IP address, and path, etc. */
	    error = net_getparams(netdev_sock);
	    if (error) {
				/* getparams makes its own noise */
		netif_close(netdev_sock);
		netdev_sock = -1;
		return (error);
	    }
	}
	if (debug)
	    printf("net_open: got rootip %s\n", inet_ntoa(rootip));

	/*
	 * Get the NFS file handle (mount).
	 */
	error = nfs_mount(netdev_sock, rootip, rootpath);
	if (error) {
	    netif_close(netdev_sock);
	    netdev_sock = -1;
	    printf("net_open: error with nfs mount 0x%x\n", error);
	    return error;
	}

	if (debug)
	    printf("root addr=%s path=%s\n", inet_ntoa(rootip), rootpath);

	netdev_opens++;
    }
    netdev_opens++;
    f->f_devdata = &netdev_sock;
    return (error);
}

int
net_close(struct open_file *f)
{

#ifdef	NETIF_DEBUG
    if (debug)
	printf("net_close: opens=%d\n", netdev_opens);
#endif

    /* On last close, do netif close, etc. */
    f->f_devdata = NULL;
    /* Extra close call? */
    if (netdev_opens <= 0)
	return (0);
    netdev_opens--;
    /* Not last close? */
    if (netdev_opens > 0)
	return(0);
    rootip.s_addr = 0;
    if (netdev_sock >= 0) {
	if (debug)
	    printf("net_close: calling netif_close()\n");
	netif_close(netdev_sock);
	netdev_sock = -1;
    }
    return (0);
}

int
net_strategy(void *devdata, int rw, daddr_t blk, size_t size, void *buf, size_t *rsize)
{
	return EIO;
}

int
net_ioctl(struct open_file *f, u_long cmd, void *data)
{
    return EIO;
}

#define SUPPORT_BOOTP

/*
 * Get info for NFS boot: our IP address, our hostname,
 * server IP address, and our root path on the server.
 * There are two ways to do this:  The old, Sun way,
 * and the more modern, BOOTP way. (RFC951, RFC1048)
 *
 * The default is to use the Sun bootparams RPC
 * (because that is what the kernel will do).
 * MD code can make try_bootp initialied data,
 * which will override this common definition.
 */
#ifdef	SUPPORT_BOOTP
int try_bootp = 1;
#endif

static int
net_getparams(int sock)
{
    char buf[MAXHOSTNAMELEN];
    char temp[FNAME_SIZE];
    char num[8];

    struct iodesc *d;
    int i;
    n_long smask;

#ifdef	SUPPORT_BOOTP
    /*
     * Try to get boot info using BOOTP.  If we succeed, then
     * the server IP address, gateway, and root path will all
     * be initialized.  If any remain uninitialized, we will
     * use RARP and RPC/bootparam (the Sun way) to get them.
     */
    if (try_bootp)
	bootp(sock);
    if (myip.s_addr != 0)
	goto exit;
    if (debug)
	printf("net_open: BOOTP failed, trying RARP/RPC...\n");
#endif

    /*
     * Use RARP to get our IP address.  This also sets our
     * netmask to the "natural" default for our address.
     */
    if (rarp_getipaddress(sock)) {
	printf("net_open: RARP failed\n");
	return (EIO);
    }
    printf("net_open: client addr: %s\n", inet_ntoa(myip));

    /* Get our hostname, server IP address, gateway. */
    if (bp_whoami(sock)) {
	printf("net_open: bootparam/whoami RPC failed\n");
	return (EIO);
    }
    printf("net_open: client name: %s\n", hostname);

    /*
     * Ignore the gateway from whoami (unreliable).
     * Use the "gateway" parameter instead.
     */
    smask = 0;
    gateip.s_addr = 0;
    if (bp_getfile(sock, "gateway", &gateip, buf) == 0) {
	/* Got it!  Parse the netmask. */
	smask = inet_addr(buf);
    }
    if (smask) {
	netmask = smask;
	if (debug)
	    printf("net_open: subnet mask: %s\n", intoa(netmask));
    }
    if (gateip.s_addr)
	if (debug)
	    printf("net_open: net gateway: %s\n", inet_ntoa(gateip));

    /* Get the root server and pathname. */
    if (bp_getfile(sock, "root", &rootip, rootpath)) {
	printf("net_open: bootparam/getfile RPC failed\n");
	return (EIO);
    }
 exit:
    /*  
     * If present, strip the server's address off of the rootpath
     * before passing it along.  This allows us to be compatible with
     * the kernel's diskless (BOOTP_NFSROOT) booting conventions
     */
    for (i = 0; i < FNAME_SIZE && rootpath[i] != '\0'; i++)
	    if (rootpath[i] == ':')
		    break;
    if (i && i != FNAME_SIZE && rootpath[i] == ':') {
	    rootpath[i++] = '\0';
	    if (inet_addr(&rootpath[0]) != INADDR_NONE)
		    rootip.s_addr = inet_addr(&rootpath[0]);
	    memcpy(&temp[0], &rootpath[i], strlen(&rootpath[i])+1);
	    memcpy(&rootpath[0], &temp[0], strlen(&rootpath[i])+1);	    
    }

    if (debug) {
	printf("net_open: server addr: %s\n", inet_ntoa(rootip));
	printf("net_open: server path: %s\n", rootpath);	    
    }

    /* do equivalent of
     *   snprintf(temp, sizeof(temp), "%6D", d->myea, ":");
     * in lame way since snprintf seems to understand "%x", but not "%x:%x"
     */
    d = socktodesc(sock);
    memset(temp, '\0', sizeof(temp));

    for (i = 0; i < ETHER_ADDR_LEN; i++) {
	if (d->myea[i] < 0x10)
	    strncat(temp, "0", 1);

	snprintf(num, sizeof(num), "%x", d->myea[i]);
	strncat(temp, num, 2);

	if (i < ETHER_ADDR_LEN-1)
	    strncat(temp, ":", 1);
    }

    setenv("boot.netif.ip", inet_ntoa(myip), 1);
    setenv("boot.netif.netmask", intoa(netmask), 1);
    setenv("boot.netif.gateway", inet_ntoa(gateip), 1);
    setenv("boot.netif.hwaddr", temp, 1);
    setenv("boot.nfsroot.server", inet_ntoa(rootip), 1);
    setenv("boot.nfsroot.path", rootpath, 1);

    return (0);
}
