/* $NetBSD: ralink_pci.c,v 1.1 2014/04/29 17:21:24 matt Exp $ */
/*-
 * Copyright (c) 2014 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by Matt Thomas of 3am Software Foundry.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include "opt_pci.h"

#include <sys/cdefs.h>

__KERNEL_RCSID(1, "$NetBSD: ralink_pci.c,v 1.1 2014/04/29 17:21:24 matt Exp $");

#include <sys/param.h>

#include <sys/bus.h>
#include <sys/device.h>

#include <dev/pci/pcireg.h>
#include <dev/pci/pcivar.h>

struct ralink_pci_softc {
	device_t sc_dev;
};

static int ralink_pci_match(device_t, cfdata_t, void *);
static void ralink_pci_attach(device_t, device_t, void *);

CFATTACH_DECL_NEW(ralink_pci, sizeof(struct ralink_pci_softc),
    ralink_pci_match, ralink_pci_attach, NULL, NULL);

static int
ralink_pci_match(device_t parent, cfdata_t cf, void *aux)
{
	return 1;
}

static void
ralink_pci_attach(device_t parent, device_t self, void *aux)
{
	struct ralink_pci_softc * const sc = device_private(self);

	sc->sc_dev = self;

	aprint_naive(": Ralink PCIe Controller\n");
	aprint_normal(": Ralink PCIe Controller\n");
}

