/*	$NetBSD: marvellvar.h,v 1.4 2015/05/03 14:38:10 hsuenaga Exp $ */
/*
 * Copyright (c) 2007 KIYOHARA Takashi
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _EVBARM_MARVELLVAR_H_
#define _EVBARM_MARVELLVAR_H_
#ifndef _LOCORE
#include <machine/bus_defs.h>
#endif

/*
 * Logical mapping for onboard/integrated peripherals
 * that are used while bootstrapping.
 *
 * u-boot sets Internal Registers to 0xf1000000.
 */
#define MARVELL_PEXMEM_VBASE			0xe0000000
#define MARVELL_INTERREGS_VBASE			0xf1000000
#define MARVELL_PEXIO_VBASE			0xf2000000

#ifndef _LOCORE
extern bus_addr_t marvell_interregs_pbase;
#endif

#endif /* _EVBARM_MARVELLVAR_H_ */
