/*	$NetBSD: vmparam.h,v 1.1 2006/09/01 21:26:18 uwe Exp $	*/

#ifndef _LANDISK_VMPARAM_H_
#define _LANDISK_VMPARAM_H_

#include <sh3/vmparam.h>

#define VM_PHYSSEG_MAX		1

#define VM_NFREELIST		1
#define VM_FREELIST_DEFAULT	0

#endif /* _LANDISK_VMPARAM_H_ */
