/*	$NetBSD: types.h,v 1.13 2021/04/01 04:43:00 simonb Exp $	*/

#include <mips/types.h>

/* MIPS specific options */

#define	__HAVE_BOOTINFO_H
#define	__HAVE_LEGACY_INTRCNT
#define	__HAVE_MIPS_MACHDEP_CACHE_CONFIG
