/*	$NetBSD: atomic_is_lock_free.c,v 1.3 2023/04/03 16:45:46 riastradh Exp $	*/

/*-
 * Copyright (c) 2023 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/cdefs.h>
__RCSID("$NetBSD: atomic_is_lock_free.c,v 1.3 2023/04/03 16:45:46 riastradh Exp $");

#include <sys/types.h>

#include <sys/stdbool.h>

/*
 * XXX Work around clang's built-in __atomic_is_lock_free by having the
 * C name be different but using an asm rename to define the right
 * symbol.
 */
bool __atomic_is_lock_free_hack(size_t, const volatile void *)
    __RENAME(__atomic_is_lock_free);
#define	__atomic_is_lock_free	__atomic_is_lock_free_hack

bool
__atomic_is_lock_free(size_t n, const volatile void *p __unused)
{

	switch (n) {
	case 1:
		return __atomic_always_lock_free(1, 0);
	case 2:
		return __atomic_always_lock_free(2, 0);
	case 4:
		return __atomic_always_lock_free(4, 0);
	case 8:
		return __atomic_always_lock_free(8, 0);
	case 16:
		return __atomic_always_lock_free(16, 0);
	default:
		return false;
	}
}
