/*	$NetBSD: cpufunc.h,v 1.1.8.1 2026/04/02 18:13:22 martin Exp $	*/

/*-
 * Copyright (c) 2023 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by Nick Hudson
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _RISCV_CPUFUNC_H_
#define _RISCV_CPUFUNC_H_

#ifdef _KERNEL

#include <sys/cdefs.h>
__KERNEL_RCSID(0, "$NetBSD: cpufunc.h,v 1.1.8.1 2026/04/02 18:13:22 martin Exp $");

#include <sys/param.h>

#include <sys/bus.h>
#include <sys/cpu.h>

int set_cpufuncs(void);

/* cache op */
#define cpu_dcache_wbinv_all()		__nothing
#define cpu_dcache_inv_all()		__nothing
#define cpu_dcache_wb_all()		__nothing
#define cpu_idcache_wbinv_all()		__nothing
#define cpu_icache_sync_all()		__nothing
#define cpu_icache_inv_all()		__nothing

#define cpu_idcache_wbinv_range(v,s)	__nothing
#define cpu_icache_sync_range(v,s)	__nothing

extern void (*cpu_dcache_wbinv_range)(vaddr_t, vsize_t);
extern void (*cpu_dcache_inv_range)(vaddr_t, vsize_t);
extern void (*cpu_dcache_wb_range)(vaddr_t, vsize_t);

extern void (*cpu_sdcache_wbinv_range)(vaddr_t, paddr_t, psize_t);
extern void (*cpu_sdcache_inv_range)(vaddr_t, paddr_t, psize_t);
extern void (*cpu_sdcache_wb_range)(vaddr_t, paddr_t, psize_t);

extern u_int   riscv_dcache_align;
extern u_int   riscv_dcache_align_mask;

#endif /* _KERNEL */

#endif /* _RISCV_CPUFUNC_H_ */
