|#
|# $NetBSD: copyright.S,v 1.3 2000/11/30 21:00:51 scw Exp $
|#

 .ascii "MOTOROLA MICROPROCESSOR & MEMORY TECHNOLOGY GROUP\n"
 .ascii "M68000 Hi-Performance Microprocessor Division\n"
 .ascii "M68060 Software Package Production Release \n"
 .ascii "\n"
 .ascii "M68060 Software Package Copyright (C) 1993, 1994, "
 .ascii "1995, 1996 Motorola Inc.\n"
 .ascii "All rights reserved.\n"
 .ascii "\n"
 .ascii "THE SOFTWARE is provided on an \"AS IS\" basis and without"
 .ascii " warranty.\n"
 .ascii "To the maximum extent permitted by applicable law,\n"
 .ascii "MOTOROLA DISCLAIMS ALL WARRANTIES WHETHER EXPRESS OR IMPLIED,\n"
 .ascii "INCLUDING IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS\n"
 .ascii "FOR A PARTICULAR PURPOSE and any warranty against infringement with\n"
 .ascii "regard to the SOFTWARE (INCLUDING ANY MODIFIED VERSIONS THEREOF)\n"
 .ascii "and any accompanying written materials. \n"
 .ascii "\n"
 .ascii "To the maximum extent permitted by applicable law,\n"
 .ascii "IN NO EVENT SHALL MOTOROLA BE LIABLE FOR ANY DAMAGES WHATSOEVER\n"
 .ascii "(INCLUDING WITHOUT LIMITATION, DAMAGES FOR LOSS OF"
 .ascii " BUSINESS PROFITS,\n"
 .ascii "BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR OTHER"
 .ascii " PECUNIARY LOSS)\n"
 .ascii "ARISING OF THE USE OR INABILITY TO USE THE SOFTWARE.\n"
 .ascii "\n"
 .ascii "Motorola assumes no responsibility for the maintenance and support\n"
 .ascii "of the SOFTWARE.  \n"
 .ascii "\n"
 .ascii "You are hereby granted a copyright license to use, modify, and"
 .ascii " distribute the\n"
 .ascii "SOFTWARE so long as this entire notice is retained without"
 .ascii " alteration\n"
 .ascii "in any modified and/or redistributed versions, and that such"
 .ascii " modified\n"
 .ascii "versions are clearly identified as such.\n"
 .ascii "No licenses are granted by implication, estoppel or otherwise"
 .ascii " under any\n"
 .ascii "patents or trademarks of Motorola, Inc.\n"
 .byte 0
 .even
