/*	$NetBSD: types.h,v 1.13 2021/01/23 19:38:53 christos Exp $	*/

#ifndef _OFPPC_TYPES_H_
#define	_OFPPC_TYPES_H_

#define	__HAVE_NEW_STYLE_BUS_H
#define	__HAVE_COMPAT_NETBSD32
#define	__HAVE_BUS_SPACE_8

#include <powerpc/types.h>

#endif
