/*      $NetBSD: bswap.h,v 1.1 2014/09/03 19:34:26 matt Exp $      */

#ifndef _OR1K_BSWAP_H_
#define _OR1K_BSWAP_H_

#include <or1k/byte_swap.h>

#define __BSWAP_RENAME
#include <sys/bswap.h>

#endif /* _OR1K_BSWAP_H_ */
