/*	$NetBSD: types.h,v 1.6 2021/04/01 04:35:46 simonb Exp $	*/
#ifndef _HPCSH_TYPES_H_
#define	_HPCSH_TYPES_H_

#include <sh3/types.h>

#define	__HAVE_DEVICE_REGISTER
#define	__HAVE_BUS_SPACE_8

#endif /* _HPCSH_TYPES_H_ */
