#! /bin/sh -
#
# $NetBSD: newvers_rm.sh,v 1.3 2024/02/08 19:28:42 christos Exp $
#
# Copyright (c) 2000 The NetBSD Foundation, Inc.
# All rights reserved.
#
# This code is derived from software contributed to The NetBSD Foundation
# by Jaromir Dolecek.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
# ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
# BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

# Script for generating of rmvers.c file from given template.
# Used for generating a RISC OS module header.
#
# Called as:
#	sh newvers_rm.sh VERSION_FILE MODULENAME

set -e

r=$(awk -F: '$1 ~ /^[0-9.]*$/ { it = $1; } END { print it }' $1)
r=$(echo $r | sed 's/\.\([0-9]\)$/.0\1/')


if [ -n "$MKREPRO_TIMESTAMP" ]; then
	# Try NetBSD date, fall back to GNU date.
	t=$(date -u -r "$MKREPRO_TIMESTAMP" '+%d %b %Y' 2> /dev/null || \
	    date -u -d "@$MKREPRO_TIMESTAMP" '+%d %b %Y')
else
       t=$(date +"%d %b %Y")
fi

echo "const char rmhelp[] = \"$2\\t${r} (${t})\";" > rmvers.c

