/* $NetBSD: types.h,v 1.16 2017/01/26 15:55:09 christos Exp $ */

#ifndef _HPCARM_TYPES_H_
#define	_HPCARM_TYPES_H_

#include <arm/arm32/types.h>

#define	__HAVE_DEVICE_REGISTER
#define	__HAVE_COMPAT_NETBSD32
#define	__GENERIC_SOFT_INTERRUPTS_ALL_LEVELS

#endif
