/* -*-C++-*-	$NetBSD: sh.h,v 1.4 2008/04/28 20:23:20 martin Exp $	*/

/*-
 * Copyright (c) 2001, 2002 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by UCHIYAMA Yasushi.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef _HPCBOOT_SH_CPU_SH_H_
#define	_HPCBOOT_SH_CPU_SH_H_

/*
 * SH3, SH4 common defines.
 */

/*
 * Address space.
 */
#define	SH_P0_START		0x00000000
#define	SH_P1_START		0x80000000
#define	SH_P2_START		0xa0000000
#define	SH_P3_START		0xc0000000
#define	SH_P4_START		0xe0000000

/*
 * Chip select.
 */
#define	SH_AREA_SIZE		0x04000000

#define	SH_AREA0_START		0x00000000
#define	SH_AREA1_START		0x04000000
#define	SH_AREA2_START		0x08000000
#define	SH_AREA3_START		0x0c000000
#define	SH_AREA4_START		0x10000000
#define	SH_AREA5_START		0x14000000
#define	SH_AREA6_START		0x18000000
#define	SH_AREA7_START		0x1c000000


#endif /* _HPCBOOT_SH_CPU_SH_H_ */
