/*	$NetBSD: types.h,v 1.26 2017/01/26 15:55:10 christos Exp $	*/

#ifndef _PMAX_TYPES_H_
#define	_PMAX_TYPES_H_

#include <mips/types.h>

#define	__HAVE_DEVICE_REGISTER
#define	__HAVE_COMPAT_NETBSD32

/* MIPS specific options */
#define	__HAVE_BOOTINFO_H
#define	__HAVE_MIPS_MACHDEP_CACHE_CONFIG

#endif
