/* $NetBSD: types.h,v 1.8 2017/01/26 15:55:08 christos Exp $ */

#ifndef _ACORN32_TYPES_H_
#define	_ACORN32_TYPES_H_

#include <arm/arm32/types.h>

#define	__HAVE_DEVICE_REGISTER
#define	__HAVE_COMPAT_NETBSD32
#endif
