/* $NetBSD: byte_swap_2.S,v 1.3 2008/02/16 17:37:13 apb Exp $ */

/*
 * Copyright (c) 1996 Carnegie-Mellon University.
 * All rights reserved.
 *
 * Author: Chris G. Demetriou
 *
 * Permission to use, copy, modify and distribute this software and
 * its documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * CARNEGIE MELLON ALLOWS FREE USE OF THIS SOFTWARE IN ITS "AS IS"
 * CONDITION.  CARNEGIE MELLON DISCLAIMS ANY LIABILITY OF ANY KIND
 * FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * Carnegie Mellon requests users of this software to return to
 *
 *  Software Distribution Coordinator  or  Software.Distribution@CS.CMU.EDU
 *  School of Computer Science
 *  Carnegie Mellon University
 *  Pittsburgh PA 15213-3890
 *
 * any improvements or extensions that they make and grant Carnegie the
 * rights to redistribute these changes.
 */

#include <machine/asm.h>

/*
 * Byte-swap a 2-byte quantity.  (Convert 0x0123 to 0x2301.)
 *
 * Argument is an unsigned 2-byte integer (uint16_t).
 */
#if defined(_KERNEL) || defined(_STANDALONE)
#define	BSWAP16	bswap16
#else /* defined(_KERNEL) || defined(_STANDALONE) */
#define	BSWAP16	__bswap16
#endif /* defined(_KERNEL) || defined(_STANDALONE) */
LEAF(BSWAP16, 1)		/* a0 contains 0x0123 */
XLEAF(htons, 1)
XLEAF(ntohs, 1)
	insbl	a0, 1, t0	/* t0 = 0x23   */
	extbl	a0, 1, t1	/* t1 = 0x  01 */
	or	t0, t1, v0	/* v0 = 0x2301 */
	RET
END(BSWAP16)
