/* $NetBSD: bus_defs.h,v 1.2 2015/06/27 03:34:26 matt Exp $ */

#ifndef _EVBMIPS_BUS_DEFS_H_
#define _EVBMIPS_BUS_DEFS_H_

#define _MIPS_NEED_BUS_DMA_BOUNCE

#include <mips/bus_space_defs.h>
#include <mips/bus_dma_defs.h>

#endif /* _EVBMIPS_BUS_DEFS_H_ */
