/* $NetBSD: bus_funcs.h,v 1.1 2011/07/01 17:10:01 dyoung Exp $ */

#ifndef _SBMIPS_BUS_FUNCS_H_
#define	_SBMIPS_BUS_FUNCS_H_

#include <mips/bus_space_funcs.h>
#include <mips/bus_dma_funcs.h>

#endif /* _SBMIPS_BUS_FUNCS_H_ */
