//{{NO_DEPENDENCIES}}
// Microsoft Developer Studio generated include file.
// Used by pbsdboot.rc
//
#define IDCSAVE                         3
#define IDD_LOAD                        101
#define IDD_FB                          103
#define IDD_MAIN_240X320                105
#define IDD_MAIN_640X240                106
#define IDD_MAIN_320X240                106
#define IDD_ABOUT                       107
#define IDB_PBSD                        109
#define IDD_PROP                        110
#define IDC_PROGRESS                    1000
#define IDC_FB_NAME                     1001
#define IDC_FB_ADDRSPIN                 1002
#define IDC_FB_ADDR                     1003
#define IDC_FB_WIDTH                    1005
#define IDC_FB_CPU                      1006
#define IDC_FB_HEIGHT                   1007
#define IDC_FB_LINEBYTES                1008
#define IDC_FB_TYPE                     1009
#define IDC_FB_MACHINE                  1010
#define IDC_KERNEL                      1015
#define IDC_FBSELECT                    1016
#define IDC_FBSETTING                   1017
#define IDC_BOOT                        1018
#define IDC_OPTIONS                     1019
#define IDC_ABOUT_EDIT                  1022
#define IDC_BITMAP                      1023
#define IDC_ABOUT_BITMAP                1023
#define IDC_PAUSE                       1024
#define IDC_STATUS                      1025
#define IDC_DEBUG                       1026
#define IDC_COMM                        1027
#define IDC_XXX                         1028
#define IDC_REVERSEVIDEO                1032
#define IDC_PROPERTY                    1033
#define IDPROPOK                        1035
#define IDC_BOOT_TIME                   1041
#define IDC_AUTOBOOT                    1042

// Next default values for new objects
// 
#ifdef APSTUDIO_INVOKED
#ifndef APSTUDIO_READONLY_SYMBOLS
#define _APS_NEXT_RESOURCE_VALUE        111
#define _APS_NEXT_COMMAND_VALUE         40001
#define _APS_NEXT_CONTROL_VALUE         1043
#define _APS_NEXT_SYMED_VALUE           101
#endif
#endif
