/*	$NetBSD: coff_machdep.h,v 1.1 1999/09/13 10:30:30 itojun Exp $	*/
#ifndef _EVBSH3_COFF_MACHDEP_H_
#define _EVBSH3_COFF_MACHDEP_H_

#include <sh3/coff_machdep.h>

#endif /* _EVBSH3_COFF_MACHDEP_H_ */
