/*	$NetBSD: mpbiosvar.h,v 1.6 2003/03/04 22:19:32 fvdl Exp $	*/

#ifndef _I386_MPBIOSVAR_H
#define _I386_MPBIOSVAR_H

/*
 * The i386 port deals with EISA busses.
 */
#define X86_MPBIOS_SUPPORT_EISA

#include <x86/mpbiosvar.h>

#endif /* _I386_MPBIOSVAR_H */
