/*	$NetBSD: param.h,v 1.3 2008/03/16 19:54:11 he Exp $	*/

#ifndef _IYONIX_PARAM_H_
#define _IYONIX_PARAM_H_

#define	_MACHINE	iyonix
#define	MACHINE		"iyonix"

#include <arm/arm32/param.h>

#endif /* _IYONIX_PARAM_H_ */
