/*	$NetBSD: types.h,v 1.15 2021/04/01 04:35:45 simonb Exp $	*/

#ifndef _CATS_TYPES_H_
#define	_CATS_TYPES_H_

#define	__HAVE_FAST_SOFTINTS
#include <arm/arm32/types.h>
#define	__HAVE_NEW_STYLE_BUS_H
#define	__HAVE_COMPAT_NETBSD32

#endif
