/*	$NetBSD: pcb.h,v 1.35 2025/04/28 13:01:27 riastradh Exp $	*/

/*
 * Copyright (c) 1998 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by Charles M. Hannum.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * Copyright (c) 1990 The Regents of the University of California.
 * All rights reserved.
 *
 * This code is derived from software contributed to Berkeley by
 * William Jolitz.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the University nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	@(#)pcb.h	5.10 (Berkeley) 5/12/91
 */

/*
 * XXXfvdl these copyrights don't really match anymore
 */

#ifndef _AMD64_PCB_H_
#define _AMD64_PCB_H_

#ifdef __x86_64__

#include <x86/cpu_extended_state.h>
#include <amd64/reg.h>

#define	NIOPORTS	1024		/* # of ports we allow to be mapped */

/*
 * IMPORTANT NOTE: this structure, including the variable-sized FPU state at
 * the end, must fit within one page.
 */
struct pcb {
	int	  pcb_flags;
#define	PCB_COMPAT32	0x01
#define	PCB_DBREGS	0x02
	u_int	  pcb_cr0;		/* saved image of CR0 */
	uint64_t pcb_rsp0;
	uint64_t pcb_cr2;		/* page fault address (CR2) */
	uint64_t pcb_cr3;
	uint64_t pcb_rsp;
	uint64_t pcb_rbp;
	void     *pcb_onfault;		/* copyin/out fault recovery */
	uint64_t  pcb_fs;
	uint64_t  pcb_gs;
	struct dbreg *pcb_dbregs;
	uint16_t pcb_fpu_dflt_cw;
	int pcb_iopl;
#ifdef XENPV			/* XXX XENPV PR kern/59371 */
	uint32_t pcb_unused[8];		/* unused */
	union savefpu	pcb_savefpu[1] __aligned(64);
#define	pcb_savefpusmall	pcb_savefpu
#else
	union savefpu *pcb_savefpu;

	uint32_t pcb_unused[6];		/* unused */

	/* fpu state, if it fits; otherwise allocated separately */
	union savefpu	pcb_savefpusmall __aligned(64);
#endif
	/* **** DO NOT ADD ANYTHING HERE **** */
};
#ifndef __lint__
#include <sys/stddef.h>		/* for offsetof() */

__CTASSERT(offsetof(struct pcb, pcb_savefpusmall) == 128);
__CTASSERT(sizeof(struct pcb) - sizeof (union savefpu) ==  128);
#endif

#else	/*	__x86_64__	*/

#include <i386/pcb.h>

#endif	/*	__x86_64__	*/

#endif /* _AMD64_PCB_H_ */
