/*	$NetBSD: bus_funcs.h,v 1.1 2011/07/01 17:10:00 dyoung Exp $	*/

#ifndef _OFPPC_BUS_FUNCS_H_
#define _OFPPC_BUS_FUNCS_H_

#include <powerpc/ofw_bus_funcs.h>
#include <powerpc/bus_funcs.h>

#endif /* _OFPPC_BUS_FUNCS_H_ */
