/*	$NetBSD: platid_mask.h,v 1.25 2011/05/18 12:11:46 nonaka Exp $	*/

/*-
 * Copyright (c) 1999-2001
 *         Shin Takemura and PocketBSD Project. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *        This product includes software developed by the NetBSD
 *        Foundation, Inc. and its contributors.
 * 4. Neither the name of The NetBSD Foundation nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
/*
 *  Do not edit.
 *  This file is automatically generated by platid.awk.
 */
#ifdef hpcmips
extern platid_t platid_mask_CPU_MIPS;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_MIPS ((int)&platid_mask_CPU_MIPS)
#endif
extern platid_t platid_mask_CPU_MIPS_VR;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_MIPS_VR ((int)&platid_mask_CPU_MIPS_VR)
#endif
extern platid_t platid_mask_CPU_MIPS_VR_41XX;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_MIPS_VR_41XX ((int)&platid_mask_CPU_MIPS_VR_41XX)
#endif
extern platid_t platid_mask_CPU_MIPS_VR_4102;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_MIPS_VR_4102 ((int)&platid_mask_CPU_MIPS_VR_4102)
#endif
extern platid_t platid_mask_CPU_MIPS_VR_4111;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_MIPS_VR_4111 ((int)&platid_mask_CPU_MIPS_VR_4111)
#endif
extern platid_t platid_mask_CPU_MIPS_VR_4121;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_MIPS_VR_4121 ((int)&platid_mask_CPU_MIPS_VR_4121)
#endif
extern platid_t platid_mask_CPU_MIPS_VR_4181;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_MIPS_VR_4181 ((int)&platid_mask_CPU_MIPS_VR_4181)
#endif
extern platid_t platid_mask_CPU_MIPS_VR_4122;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_MIPS_VR_4122 ((int)&platid_mask_CPU_MIPS_VR_4122)
#endif
extern platid_t platid_mask_CPU_MIPS_VR_4131;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_MIPS_VR_4131 ((int)&platid_mask_CPU_MIPS_VR_4131)
#endif
extern platid_t platid_mask_CPU_MIPS_VR_4181A;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_MIPS_VR_4181A ((int)&platid_mask_CPU_MIPS_VR_4181A)
#endif
extern platid_t platid_mask_CPU_MIPS_TX;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_MIPS_TX ((int)&platid_mask_CPU_MIPS_TX)
#endif
extern platid_t platid_mask_CPU_MIPS_TX_3900;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_MIPS_TX_3900 ((int)&platid_mask_CPU_MIPS_TX_3900)
#endif
extern platid_t platid_mask_CPU_MIPS_TX_3911;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_MIPS_TX_3911 ((int)&platid_mask_CPU_MIPS_TX_3911)
#endif
extern platid_t platid_mask_CPU_MIPS_TX_3912;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_MIPS_TX_3912 ((int)&platid_mask_CPU_MIPS_TX_3912)
#endif
extern platid_t platid_mask_CPU_MIPS_TX_3920;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_MIPS_TX_3920 ((int)&platid_mask_CPU_MIPS_TX_3920)
#endif
extern platid_t platid_mask_CPU_MIPS_TX_3922;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_MIPS_TX_3922 ((int)&platid_mask_CPU_MIPS_TX_3922)
#endif
extern platid_t platid_mask_CPU_MIPS_TX_3927;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_MIPS_TX_3927 ((int)&platid_mask_CPU_MIPS_TX_3927)
#endif
#endif /* hpcmips */
#ifdef hpcsh
extern platid_t platid_mask_CPU_SH;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_SH ((int)&platid_mask_CPU_SH)
#endif
extern platid_t platid_mask_CPU_SH_3;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_SH_3 ((int)&platid_mask_CPU_SH_3)
#endif
extern platid_t platid_mask_CPU_SH_3_7709;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_SH_3_7709 ((int)&platid_mask_CPU_SH_3_7709)
#endif
extern platid_t platid_mask_CPU_SH_3_7709A;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_SH_3_7709A ((int)&platid_mask_CPU_SH_3_7709A)
#endif
extern platid_t platid_mask_CPU_SH_3_7707;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_SH_3_7707 ((int)&platid_mask_CPU_SH_3_7707)
#endif
extern platid_t platid_mask_CPU_SH_4;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_SH_4 ((int)&platid_mask_CPU_SH_4)
#endif
extern platid_t platid_mask_CPU_SH_4_7750;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_SH_4_7750 ((int)&platid_mask_CPU_SH_4_7750)
#endif
#endif /* hpcsh */
#ifdef hpcarm
extern platid_t platid_mask_CPU_ARM;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_ARM ((int)&platid_mask_CPU_ARM)
#endif
extern platid_t platid_mask_CPU_ARM_STRONGARM;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_ARM_STRONGARM ((int)&platid_mask_CPU_ARM_STRONGARM)
#endif
extern platid_t platid_mask_CPU_ARM_STRONGARM_SA1100;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_ARM_STRONGARM_SA1100 ((int)&platid_mask_CPU_ARM_STRONGARM_SA1100)
#endif
extern platid_t platid_mask_CPU_ARM_STRONGARM_SA1110;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_ARM_STRONGARM_SA1110 ((int)&platid_mask_CPU_ARM_STRONGARM_SA1110)
#endif
extern platid_t platid_mask_CPU_ARM_XSCALE;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_ARM_XSCALE ((int)&platid_mask_CPU_ARM_XSCALE)
#endif
extern platid_t platid_mask_CPU_ARM_XSCALE_PXA250;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_ARM_XSCALE_PXA250 ((int)&platid_mask_CPU_ARM_XSCALE_PXA250)
#endif
extern platid_t platid_mask_CPU_ARM_XSCALE_PXA270;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define GENERIC_ARM_XSCALE_PXA270 ((int)&platid_mask_CPU_ARM_XSCALE_PXA270)
#endif
#endif /* hpcarm */
#ifdef hpcmips
extern platid_t platid_mask_MACH_NEC;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC ((int)&platid_mask_MACH_NEC)
#endif
extern platid_t platid_mask_MACH_NEC_MCCS;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC_MCCS ((int)&platid_mask_MACH_NEC_MCCS)
#endif
extern platid_t platid_mask_MACH_NEC_MCCS_1X;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC_MCCS_1X ((int)&platid_mask_MACH_NEC_MCCS_1X)
#endif
extern platid_t platid_mask_MACH_NEC_MCCS_11;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC_MCCS_11 ((int)&platid_mask_MACH_NEC_MCCS_11)
#endif
extern platid_t platid_mask_MACH_NEC_MCCS_12;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC_MCCS_12 ((int)&platid_mask_MACH_NEC_MCCS_12)
#endif
extern platid_t platid_mask_MACH_NEC_MCCS_13;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC_MCCS_13 ((int)&platid_mask_MACH_NEC_MCCS_13)
#endif
extern platid_t platid_mask_MACH_NEC_MCR;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC_MCR ((int)&platid_mask_MACH_NEC_MCR)
#endif
extern platid_t platid_mask_MACH_NEC_MCR_3XX;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC_MCR_3XX ((int)&platid_mask_MACH_NEC_MCR_3XX)
#endif
extern platid_t platid_mask_MACH_NEC_MCR_300;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC_MCR_300 ((int)&platid_mask_MACH_NEC_MCR_300)
#endif
extern platid_t platid_mask_MACH_NEC_MCR_320;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC_MCR_320 ((int)&platid_mask_MACH_NEC_MCR_320)
#endif
extern platid_t platid_mask_MACH_NEC_MCR_FORDOCOMO;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC_MCR_FORDOCOMO ((int)&platid_mask_MACH_NEC_MCR_FORDOCOMO)
#endif
extern platid_t platid_mask_MACH_NEC_MCR_MPRO700;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC_MCR_MPRO700 ((int)&platid_mask_MACH_NEC_MCR_MPRO700)
#endif
extern platid_t platid_mask_MACH_NEC_MCR_330;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC_MCR_330 ((int)&platid_mask_MACH_NEC_MCR_330)
#endif
extern platid_t platid_mask_MACH_NEC_MCR_5XX;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC_MCR_5XX ((int)&platid_mask_MACH_NEC_MCR_5XX)
#endif
extern platid_t platid_mask_MACH_NEC_MCR_500;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC_MCR_500 ((int)&platid_mask_MACH_NEC_MCR_500)
#endif
extern platid_t platid_mask_MACH_NEC_MCR_510;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC_MCR_510 ((int)&platid_mask_MACH_NEC_MCR_510)
#endif
extern platid_t platid_mask_MACH_NEC_MCR_520;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC_MCR_520 ((int)&platid_mask_MACH_NEC_MCR_520)
#endif
extern platid_t platid_mask_MACH_NEC_MCR_520A;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC_MCR_520A ((int)&platid_mask_MACH_NEC_MCR_520A)
#endif
extern platid_t platid_mask_MACH_NEC_MCR_500A;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC_MCR_500A ((int)&platid_mask_MACH_NEC_MCR_500A)
#endif
extern platid_t platid_mask_MACH_NEC_MCR_530;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC_MCR_530 ((int)&platid_mask_MACH_NEC_MCR_530)
#endif
extern platid_t platid_mask_MACH_NEC_MCR_430;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC_MCR_430 ((int)&platid_mask_MACH_NEC_MCR_430)
#endif
extern platid_t platid_mask_MACH_NEC_MCR_530A;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC_MCR_530A ((int)&platid_mask_MACH_NEC_MCR_530A)
#endif
extern platid_t platid_mask_MACH_NEC_MCR_SIGMARION;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC_MCR_SIGMARION ((int)&platid_mask_MACH_NEC_MCR_SIGMARION)
#endif
extern platid_t platid_mask_MACH_NEC_MCR_550;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC_MCR_550 ((int)&platid_mask_MACH_NEC_MCR_550)
#endif
extern platid_t platid_mask_MACH_NEC_MCR_450;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC_MCR_450 ((int)&platid_mask_MACH_NEC_MCR_450)
#endif
extern platid_t platid_mask_MACH_NEC_MCR_SIGMARION2;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC_MCR_SIGMARION2 ((int)&platid_mask_MACH_NEC_MCR_SIGMARION2)
#endif
extern platid_t platid_mask_MACH_NEC_MCR_7XX;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC_MCR_7XX ((int)&platid_mask_MACH_NEC_MCR_7XX)
#endif
extern platid_t platid_mask_MACH_NEC_MCR_700;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC_MCR_700 ((int)&platid_mask_MACH_NEC_MCR_700)
#endif
extern platid_t platid_mask_MACH_NEC_MCR_700A;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC_MCR_700A ((int)&platid_mask_MACH_NEC_MCR_700A)
#endif
extern platid_t platid_mask_MACH_NEC_MCR_730;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC_MCR_730 ((int)&platid_mask_MACH_NEC_MCR_730)
#endif
extern platid_t platid_mask_MACH_NEC_MCR_730A;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define NEC_MCR_730A ((int)&platid_mask_MACH_NEC_MCR_730A)
#endif
#endif /* hpcmips */
#ifdef hpcmips
extern platid_t platid_mask_MACH_EVEREX;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define EVEREX ((int)&platid_mask_MACH_EVEREX)
#endif
extern platid_t platid_mask_MACH_EVEREX_FREESTYLE;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define EVEREX_FREESTYLE ((int)&platid_mask_MACH_EVEREX_FREESTYLE)
#endif
extern platid_t platid_mask_MACH_EVEREX_FREESTYLE_AXX;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define EVEREX_FREESTYLE_AXX ((int)&platid_mask_MACH_EVEREX_FREESTYLE_AXX)
#endif
extern platid_t platid_mask_MACH_EVEREX_FREESTYLE_A10;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define EVEREX_FREESTYLE_A10 ((int)&platid_mask_MACH_EVEREX_FREESTYLE_A10)
#endif
extern platid_t platid_mask_MACH_EVEREX_FREESTYLE_A15;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define EVEREX_FREESTYLE_A15 ((int)&platid_mask_MACH_EVEREX_FREESTYLE_A15)
#endif
extern platid_t platid_mask_MACH_EVEREX_FREESTYLE_A20;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define EVEREX_FREESTYLE_A20 ((int)&platid_mask_MACH_EVEREX_FREESTYLE_A20)
#endif
#endif /* hpcmips */
extern platid_t platid_mask_MACH_CASIO;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define CASIO ((int)&platid_mask_MACH_CASIO)
#endif
#ifdef hpcmips
extern platid_t platid_mask_MACH_CASIO_CASSIOPEIAE;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define CASIO_CASSIOPEIAE ((int)&platid_mask_MACH_CASIO_CASSIOPEIAE)
#endif
extern platid_t platid_mask_MACH_CASIO_CASSIOPEIAE_EXX;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define CASIO_CASSIOPEIAE_EXX ((int)&platid_mask_MACH_CASIO_CASSIOPEIAE_EXX)
#endif
extern platid_t platid_mask_MACH_CASIO_CASSIOPEIAE_E10;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define CASIO_CASSIOPEIAE_E10 ((int)&platid_mask_MACH_CASIO_CASSIOPEIAE_E10)
#endif
extern platid_t platid_mask_MACH_CASIO_CASSIOPEIAE_E11;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define CASIO_CASSIOPEIAE_E11 ((int)&platid_mask_MACH_CASIO_CASSIOPEIAE_E11)
#endif
extern platid_t platid_mask_MACH_CASIO_CASSIOPEIAE_E15;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define CASIO_CASSIOPEIAE_E15 ((int)&platid_mask_MACH_CASIO_CASSIOPEIAE_E15)
#endif
extern platid_t platid_mask_MACH_CASIO_CASSIOPEIAE_E55;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define CASIO_CASSIOPEIAE_E55 ((int)&platid_mask_MACH_CASIO_CASSIOPEIAE_E55)
#endif
extern platid_t platid_mask_MACH_CASIO_CASSIOPEIAE_FORDOCOMO;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define CASIO_CASSIOPEIAE_FORDOCOMO ((int)&platid_mask_MACH_CASIO_CASSIOPEIAE_FORDOCOMO)
#endif
extern platid_t platid_mask_MACH_CASIO_CASSIOPEIAE_E65;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define CASIO_CASSIOPEIAE_E65 ((int)&platid_mask_MACH_CASIO_CASSIOPEIAE_E65)
#endif
extern platid_t platid_mask_MACH_CASIO_CASSIOPEIAE_EXXX;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define CASIO_CASSIOPEIAE_EXXX ((int)&platid_mask_MACH_CASIO_CASSIOPEIAE_EXXX)
#endif
extern platid_t platid_mask_MACH_CASIO_CASSIOPEIAE_E100;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define CASIO_CASSIOPEIAE_E100 ((int)&platid_mask_MACH_CASIO_CASSIOPEIAE_E100)
#endif
extern platid_t platid_mask_MACH_CASIO_CASSIOPEIAE_E105;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define CASIO_CASSIOPEIAE_E105 ((int)&platid_mask_MACH_CASIO_CASSIOPEIAE_E105)
#endif
extern platid_t platid_mask_MACH_CASIO_CASSIOPEIAE_E500;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define CASIO_CASSIOPEIAE_E500 ((int)&platid_mask_MACH_CASIO_CASSIOPEIAE_E500)
#endif
extern platid_t platid_mask_MACH_CASIO_CASSIOPEIAE_E507;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define CASIO_CASSIOPEIAE_E507 ((int)&platid_mask_MACH_CASIO_CASSIOPEIAE_E507)
#endif
extern platid_t platid_mask_MACH_CASIO_POCKETPOSTPET;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define CASIO_POCKETPOSTPET ((int)&platid_mask_MACH_CASIO_POCKETPOSTPET)
#endif
extern platid_t platid_mask_MACH_CASIO_POCKETPOSTPET_POCKETPOSTPET;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define CASIO_POCKETPOSTPET_POCKETPOSTPET ((int)&platid_mask_MACH_CASIO_POCKETPOSTPET_POCKETPOSTPET)
#endif
#endif /* hpcmips */
#ifdef hpcsh
extern platid_t platid_mask_MACH_CASIO_CASSIOPEIAA;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define CASIO_CASSIOPEIAA ((int)&platid_mask_MACH_CASIO_CASSIOPEIAA)
#endif
extern platid_t platid_mask_MACH_CASIO_CASSIOPEIAA_AXX;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define CASIO_CASSIOPEIAA_AXX ((int)&platid_mask_MACH_CASIO_CASSIOPEIAA_AXX)
#endif
extern platid_t platid_mask_MACH_CASIO_CASSIOPEIAA_A55V;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define CASIO_CASSIOPEIAA_A55V ((int)&platid_mask_MACH_CASIO_CASSIOPEIAA_A55V)
#endif
#endif /* hpcsh */
#if defined(hpcmips) || defined(hpcarm)
extern platid_t platid_mask_MACH_SHARP;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define SHARP ((int)&platid_mask_MACH_SHARP)
#endif
#if defined(hpcmips)
extern platid_t platid_mask_MACH_SHARP_TRIPAD;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define SHARP_TRIPAD ((int)&platid_mask_MACH_SHARP_TRIPAD)
#endif
extern platid_t platid_mask_MACH_SHARP_TRIPAD_PV;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define SHARP_TRIPAD_PV ((int)&platid_mask_MACH_SHARP_TRIPAD_PV)
#endif
extern platid_t platid_mask_MACH_SHARP_TRIPAD_PV6000;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define SHARP_TRIPAD_PV6000 ((int)&platid_mask_MACH_SHARP_TRIPAD_PV6000)
#endif
extern platid_t platid_mask_MACH_SHARP_TELIOS;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define SHARP_TELIOS ((int)&platid_mask_MACH_SHARP_TELIOS)
#endif
extern platid_t platid_mask_MACH_SHARP_TELIOS_HCAJ;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define SHARP_TELIOS_HCAJ ((int)&platid_mask_MACH_SHARP_TELIOS_HCAJ)
#endif
extern platid_t platid_mask_MACH_SHARP_TELIOS_HCAJ1_JP;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define SHARP_TELIOS_HCAJ1_JP ((int)&platid_mask_MACH_SHARP_TELIOS_HCAJ1_JP)
#endif
extern platid_t platid_mask_MACH_SHARP_TELIOS_HCAJ2_JP;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define SHARP_TELIOS_HCAJ2_JP ((int)&platid_mask_MACH_SHARP_TELIOS_HCAJ2_JP)
#endif
extern platid_t platid_mask_MACH_SHARP_TELIOS_HCAJ3_JP;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define SHARP_TELIOS_HCAJ3_JP ((int)&platid_mask_MACH_SHARP_TELIOS_HCAJ3_JP)
#endif
extern platid_t platid_mask_MACH_SHARP_TELIOS_HCVJ;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define SHARP_TELIOS_HCVJ ((int)&platid_mask_MACH_SHARP_TELIOS_HCVJ)
#endif
extern platid_t platid_mask_MACH_SHARP_TELIOS_HCVJ1C_JP;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define SHARP_TELIOS_HCVJ1C_JP ((int)&platid_mask_MACH_SHARP_TELIOS_HCVJ1C_JP)
#endif
extern platid_t platid_mask_MACH_SHARP_MOBILON;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define SHARP_MOBILON ((int)&platid_mask_MACH_SHARP_MOBILON)
#endif
extern platid_t platid_mask_MACH_SHARP_MOBILON_HC;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define SHARP_MOBILON_HC ((int)&platid_mask_MACH_SHARP_MOBILON_HC)
#endif
extern platid_t platid_mask_MACH_SHARP_MOBILON_HC4100;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define SHARP_MOBILON_HC4100 ((int)&platid_mask_MACH_SHARP_MOBILON_HC4100)
#endif
extern platid_t platid_mask_MACH_SHARP_MOBILON_HC4500;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define SHARP_MOBILON_HC4500 ((int)&platid_mask_MACH_SHARP_MOBILON_HC4500)
#endif
extern platid_t platid_mask_MACH_SHARP_MOBILON_HC1200;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define SHARP_MOBILON_HC1200 ((int)&platid_mask_MACH_SHARP_MOBILON_HC1200)
#endif
#endif /* hpcmips */
#if defined(hpcarm)
extern platid_t platid_mask_MACH_SHARP_WZERO3;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define SHARP_WZERO3 ((int)&platid_mask_MACH_SHARP_WZERO3)
#endif
extern platid_t platid_mask_MACH_SHARP_WZERO3_WS003SH;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define SHARP_WZERO3_WS003SH ((int)&platid_mask_MACH_SHARP_WZERO3_WS003SH)
#endif
extern platid_t platid_mask_MACH_SHARP_WZERO3_WS004SH;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define SHARP_WZERO3_WS004SH ((int)&platid_mask_MACH_SHARP_WZERO3_WS004SH)
#endif
extern platid_t platid_mask_MACH_SHARP_WZERO3_WS007SH;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define SHARP_WZERO3_WS007SH ((int)&platid_mask_MACH_SHARP_WZERO3_WS007SH)
#endif
extern platid_t platid_mask_MACH_SHARP_WZERO3_WS011SH;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define SHARP_WZERO3_WS011SH ((int)&platid_mask_MACH_SHARP_WZERO3_WS011SH)
#endif
extern platid_t platid_mask_MACH_SHARP_WZERO3_WS020SH;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define SHARP_WZERO3_WS020SH ((int)&platid_mask_MACH_SHARP_WZERO3_WS020SH)
#endif
#endif /* hpcarm */
#endif /* hpcmips || hpcarm */
#ifdef hpcmips
extern platid_t platid_mask_MACH_FUJITSU;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define FUJITSU ((int)&platid_mask_MACH_FUJITSU)
#endif
extern platid_t platid_mask_MACH_FUJITSU_INTERTOP;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define FUJITSU_INTERTOP ((int)&platid_mask_MACH_FUJITSU_INTERTOP)
#endif
extern platid_t platid_mask_MACH_FUJITSU_INTERTOP_ITXXX;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define FUJITSU_INTERTOP_ITXXX ((int)&platid_mask_MACH_FUJITSU_INTERTOP_ITXXX)
#endif
extern platid_t platid_mask_MACH_FUJITSU_INTERTOP_IT300;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define FUJITSU_INTERTOP_IT300 ((int)&platid_mask_MACH_FUJITSU_INTERTOP_IT300)
#endif
extern platid_t platid_mask_MACH_FUJITSU_INTERTOP_IT310;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define FUJITSU_INTERTOP_IT310 ((int)&platid_mask_MACH_FUJITSU_INTERTOP_IT310)
#endif
extern platid_t platid_mask_MACH_FUJITSU_PENCENTRA;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define FUJITSU_PENCENTRA ((int)&platid_mask_MACH_FUJITSU_PENCENTRA)
#endif
extern platid_t platid_mask_MACH_FUJITSU_PENCENTRA_130;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define FUJITSU_PENCENTRA_130 ((int)&platid_mask_MACH_FUJITSU_PENCENTRA_130)
#endif
extern platid_t platid_mask_MACH_FUJITSU_PENCENTRA_130TM;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define FUJITSU_PENCENTRA_130TM ((int)&platid_mask_MACH_FUJITSU_PENCENTRA_130TM)
#endif
extern platid_t platid_mask_MACH_FUJITSU_PENCENTRA_130RF;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define FUJITSU_PENCENTRA_130RF ((int)&platid_mask_MACH_FUJITSU_PENCENTRA_130RF)
#endif
extern platid_t platid_mask_MACH_FUJITSU_PENCENTRA_200;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define FUJITSU_PENCENTRA_200 ((int)&platid_mask_MACH_FUJITSU_PENCENTRA_200)
#endif
extern platid_t platid_mask_MACH_FUJITSU_PENCENTRA_200CTM;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define FUJITSU_PENCENTRA_200CTM ((int)&platid_mask_MACH_FUJITSU_PENCENTRA_200CTM)
#endif
extern platid_t platid_mask_MACH_FUJITSU_PENCENTRA_200CRF;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define FUJITSU_PENCENTRA_200CRF ((int)&platid_mask_MACH_FUJITSU_PENCENTRA_200CRF)
#endif
#endif /* hpcmips */
#ifdef hpcmips
extern platid_t platid_mask_MACH_PHILIPS;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define PHILIPS ((int)&platid_mask_MACH_PHILIPS)
#endif
extern platid_t platid_mask_MACH_PHILIPS_NINO;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define PHILIPS_NINO ((int)&platid_mask_MACH_PHILIPS_NINO)
#endif
extern platid_t platid_mask_MACH_PHILIPS_NINO_3XX;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define PHILIPS_NINO_3XX ((int)&platid_mask_MACH_PHILIPS_NINO_3XX)
#endif
extern platid_t platid_mask_MACH_PHILIPS_NINO_312;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define PHILIPS_NINO_312 ((int)&platid_mask_MACH_PHILIPS_NINO_312)
#endif
#endif /* hpcmips */
extern platid_t platid_mask_MACH_COMPAQ;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define COMPAQ ((int)&platid_mask_MACH_COMPAQ)
#endif
#ifdef hpcmips
extern platid_t platid_mask_MACH_COMPAQ_C;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define COMPAQ_C ((int)&platid_mask_MACH_COMPAQ_C)
#endif
extern platid_t platid_mask_MACH_COMPAQ_C_8XX;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define COMPAQ_C_8XX ((int)&platid_mask_MACH_COMPAQ_C_8XX)
#endif
extern platid_t platid_mask_MACH_COMPAQ_C_810;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define COMPAQ_C_810 ((int)&platid_mask_MACH_COMPAQ_C_810)
#endif
extern platid_t platid_mask_MACH_COMPAQ_C_201X;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define COMPAQ_C_201X ((int)&platid_mask_MACH_COMPAQ_C_201X)
#endif
extern platid_t platid_mask_MACH_COMPAQ_C_2010;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define COMPAQ_C_2010 ((int)&platid_mask_MACH_COMPAQ_C_2010)
#endif
extern platid_t platid_mask_MACH_COMPAQ_C_2015;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define COMPAQ_C_2015 ((int)&platid_mask_MACH_COMPAQ_C_2015)
#endif
extern platid_t platid_mask_MACH_COMPAQ_AERO;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define COMPAQ_AERO ((int)&platid_mask_MACH_COMPAQ_AERO)
#endif
extern platid_t platid_mask_MACH_COMPAQ_AERO_15XX;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define COMPAQ_AERO_15XX ((int)&platid_mask_MACH_COMPAQ_AERO_15XX)
#endif
extern platid_t platid_mask_MACH_COMPAQ_AERO_1530;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define COMPAQ_AERO_1530 ((int)&platid_mask_MACH_COMPAQ_AERO_1530)
#endif
extern platid_t platid_mask_MACH_COMPAQ_AERO_21XX;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define COMPAQ_AERO_21XX ((int)&platid_mask_MACH_COMPAQ_AERO_21XX)
#endif
extern platid_t platid_mask_MACH_COMPAQ_AERO_2110;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define COMPAQ_AERO_2110 ((int)&platid_mask_MACH_COMPAQ_AERO_2110)
#endif
extern platid_t platid_mask_MACH_COMPAQ_AERO_2130;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define COMPAQ_AERO_2130 ((int)&platid_mask_MACH_COMPAQ_AERO_2130)
#endif
extern platid_t platid_mask_MACH_COMPAQ_AERO_2140;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define COMPAQ_AERO_2140 ((int)&platid_mask_MACH_COMPAQ_AERO_2140)
#endif
extern platid_t platid_mask_MACH_COMPAQ_PRESARIO;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define COMPAQ_PRESARIO ((int)&platid_mask_MACH_COMPAQ_PRESARIO)
#endif
extern platid_t platid_mask_MACH_COMPAQ_PRESARIO_21X;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define COMPAQ_PRESARIO_21X ((int)&platid_mask_MACH_COMPAQ_PRESARIO_21X)
#endif
extern platid_t platid_mask_MACH_COMPAQ_PRESARIO_213;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define COMPAQ_PRESARIO_213 ((int)&platid_mask_MACH_COMPAQ_PRESARIO_213)
#endif
#endif /* hpcmips */
#ifdef hpcarm
extern platid_t platid_mask_MACH_COMPAQ_IPAQ;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define COMPAQ_IPAQ ((int)&platid_mask_MACH_COMPAQ_IPAQ)
#endif
extern platid_t platid_mask_MACH_COMPAQ_IPAQ_H31XX;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define COMPAQ_IPAQ_H31XX ((int)&platid_mask_MACH_COMPAQ_IPAQ_H31XX)
#endif
extern platid_t platid_mask_MACH_COMPAQ_IPAQ_H3100;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define COMPAQ_IPAQ_H3100 ((int)&platid_mask_MACH_COMPAQ_IPAQ_H3100)
#endif
extern platid_t platid_mask_MACH_COMPAQ_IPAQ_H36XX;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define COMPAQ_IPAQ_H36XX ((int)&platid_mask_MACH_COMPAQ_IPAQ_H36XX)
#endif
extern platid_t platid_mask_MACH_COMPAQ_IPAQ_H3600;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define COMPAQ_IPAQ_H3600 ((int)&platid_mask_MACH_COMPAQ_IPAQ_H3600)
#endif
extern platid_t platid_mask_MACH_COMPAQ_IPAQ_H3660;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define COMPAQ_IPAQ_H3660 ((int)&platid_mask_MACH_COMPAQ_IPAQ_H3660)
#endif
extern platid_t platid_mask_MACH_COMPAQ_IPAQ_H39XX;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define COMPAQ_IPAQ_H39XX ((int)&platid_mask_MACH_COMPAQ_IPAQ_H39XX)
#endif
extern platid_t platid_mask_MACH_COMPAQ_IPAQ_H3900;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define COMPAQ_IPAQ_H3900 ((int)&platid_mask_MACH_COMPAQ_IPAQ_H3900)
#endif
#endif /* hpcarm */
#ifdef hpcsh
extern platid_t platid_mask_MACH_COMPAQ_AERO;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define COMPAQ_AERO ((int)&platid_mask_MACH_COMPAQ_AERO)
#endif
extern platid_t platid_mask_MACH_COMPAQ_AERO_8000;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define COMPAQ_AERO_8000 ((int)&platid_mask_MACH_COMPAQ_AERO_8000)
#endif
#endif /* hpcsh */
#ifdef hpcmips
extern platid_t platid_mask_MACH_VICTOR;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define VICTOR ((int)&platid_mask_MACH_VICTOR)
#endif
extern platid_t platid_mask_MACH_VICTOR_INTERLINK;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define VICTOR_INTERLINK ((int)&platid_mask_MACH_VICTOR_INTERLINK)
#endif
extern platid_t platid_mask_MACH_VICTOR_INTERLINK_MP;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define VICTOR_INTERLINK_MP ((int)&platid_mask_MACH_VICTOR_INTERLINK_MP)
#endif
extern platid_t platid_mask_MACH_VICTOR_INTERLINK_MPC101;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define VICTOR_INTERLINK_MPC101 ((int)&platid_mask_MACH_VICTOR_INTERLINK_MPC101)
#endif
extern platid_t platid_mask_MACH_VICTOR_INTERLINK_MPC303;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define VICTOR_INTERLINK_MPC303 ((int)&platid_mask_MACH_VICTOR_INTERLINK_MPC303)
#endif
extern platid_t platid_mask_MACH_VICTOR_INTERLINK_MPC304;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define VICTOR_INTERLINK_MPC304 ((int)&platid_mask_MACH_VICTOR_INTERLINK_MPC304)
#endif
#endif /* hpcmips */
#ifdef hpcmips
extern platid_t platid_mask_MACH_IBM;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define IBM ((int)&platid_mask_MACH_IBM)
#endif
extern platid_t platid_mask_MACH_IBM_WORKPAD;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define IBM_WORKPAD ((int)&platid_mask_MACH_IBM_WORKPAD)
#endif
extern platid_t platid_mask_MACH_IBM_WORKPAD_Z50;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define IBM_WORKPAD_Z50 ((int)&platid_mask_MACH_IBM_WORKPAD_Z50)
#endif
extern platid_t platid_mask_MACH_IBM_WORKPAD_26011AU;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define IBM_WORKPAD_26011AU ((int)&platid_mask_MACH_IBM_WORKPAD_26011AU)
#endif
#endif /* hpcmips */
#ifdef hpcmips
extern platid_t platid_mask_MACH_VADEM;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define VADEM ((int)&platid_mask_MACH_VADEM)
#endif
extern platid_t platid_mask_MACH_VADEM_CLIO;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define VADEM_CLIO ((int)&platid_mask_MACH_VADEM_CLIO)
#endif
extern platid_t platid_mask_MACH_VADEM_CLIO_C;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define VADEM_CLIO_C ((int)&platid_mask_MACH_VADEM_CLIO_C)
#endif
extern platid_t platid_mask_MACH_VADEM_CLIO_C1000;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define VADEM_CLIO_C1000 ((int)&platid_mask_MACH_VADEM_CLIO_C1000)
#endif
extern platid_t platid_mask_MACH_VADEM_CLIO_C1050;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define VADEM_CLIO_C1050 ((int)&platid_mask_MACH_VADEM_CLIO_C1050)
#endif
#endif /* hpcmips */
extern platid_t platid_mask_MACH_HP;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP ((int)&platid_mask_MACH_HP)
#endif
#ifdef hpcsh
extern platid_t platid_mask_MACH_HP_LX;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_LX ((int)&platid_mask_MACH_HP_LX)
#endif
extern platid_t platid_mask_MACH_HP_LX_620;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_LX_620 ((int)&platid_mask_MACH_HP_LX_620)
#endif
extern platid_t platid_mask_MACH_HP_LX_620JP;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_LX_620JP ((int)&platid_mask_MACH_HP_LX_620JP)
#endif
extern platid_t platid_mask_MACH_HP_LX_360;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_LX_360 ((int)&platid_mask_MACH_HP_LX_360)
#endif
#endif /* hpcsh */
extern platid_t platid_mask_MACH_HP_JORNADA;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_JORNADA ((int)&platid_mask_MACH_HP_JORNADA)
#endif
#ifdef hpcsh
extern platid_t platid_mask_MACH_HP_JORNADA_6XX;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_JORNADA_6XX ((int)&platid_mask_MACH_HP_JORNADA_6XX)
#endif
extern platid_t platid_mask_MACH_HP_JORNADA_680;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_JORNADA_680 ((int)&platid_mask_MACH_HP_JORNADA_680)
#endif
extern platid_t platid_mask_MACH_HP_JORNADA_680JP;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_JORNADA_680JP ((int)&platid_mask_MACH_HP_JORNADA_680JP)
#endif
extern platid_t platid_mask_MACH_HP_JORNADA_680EU;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_JORNADA_680EU ((int)&platid_mask_MACH_HP_JORNADA_680EU)
#endif
extern platid_t platid_mask_MACH_HP_JORNADA_680DE;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_JORNADA_680DE ((int)&platid_mask_MACH_HP_JORNADA_680DE)
#endif
extern platid_t platid_mask_MACH_HP_JORNADA_690;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_JORNADA_690 ((int)&platid_mask_MACH_HP_JORNADA_690)
#endif
extern platid_t platid_mask_MACH_HP_JORNADA_690JP;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_JORNADA_690JP ((int)&platid_mask_MACH_HP_JORNADA_690JP)
#endif
extern platid_t platid_mask_MACH_HP_JORNADA_690EU;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_JORNADA_690EU ((int)&platid_mask_MACH_HP_JORNADA_690EU)
#endif
extern platid_t platid_mask_MACH_HP_JORNADA_690DE;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_JORNADA_690DE ((int)&platid_mask_MACH_HP_JORNADA_690DE)
#endif
extern platid_t platid_mask_MACH_HP_JORNADA_680FR;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_JORNADA_680FR ((int)&platid_mask_MACH_HP_JORNADA_680FR)
#endif
extern platid_t platid_mask_MACH_HP_JORNADA_690FR;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_JORNADA_690FR ((int)&platid_mask_MACH_HP_JORNADA_690FR)
#endif
extern platid_t platid_mask_MACH_HP_JORNADA_680SV;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_JORNADA_680SV ((int)&platid_mask_MACH_HP_JORNADA_680SV)
#endif
extern platid_t platid_mask_MACH_HP_JORNADA_690SV;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_JORNADA_690SV ((int)&platid_mask_MACH_HP_JORNADA_690SV)
#endif
extern platid_t platid_mask_MACH_HP_JORNADA_680ES;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_JORNADA_680ES ((int)&platid_mask_MACH_HP_JORNADA_680ES)
#endif
extern platid_t platid_mask_MACH_HP_JORNADA_690ES;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_JORNADA_690ES ((int)&platid_mask_MACH_HP_JORNADA_690ES)
#endif
#endif /* hpcsh */
#ifdef hpcarm
extern platid_t platid_mask_MACH_HP_JORNADA_7XX;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_JORNADA_7XX ((int)&platid_mask_MACH_HP_JORNADA_7XX)
#endif
extern platid_t platid_mask_MACH_HP_JORNADA_720;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_JORNADA_720 ((int)&platid_mask_MACH_HP_JORNADA_720)
#endif
extern platid_t platid_mask_MACH_HP_JORNADA_720JP;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_JORNADA_720JP ((int)&platid_mask_MACH_HP_JORNADA_720JP)
#endif
extern platid_t platid_mask_MACH_HP_JORNADA_720EU;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_JORNADA_720EU ((int)&platid_mask_MACH_HP_JORNADA_720EU)
#endif
extern platid_t platid_mask_MACH_HP_JORNADA_720DE;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_JORNADA_720DE ((int)&platid_mask_MACH_HP_JORNADA_720DE)
#endif
extern platid_t platid_mask_MACH_HP_JORNADA_720FR;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_JORNADA_720FR ((int)&platid_mask_MACH_HP_JORNADA_720FR)
#endif
extern platid_t platid_mask_MACH_HP_JORNADA_720SV;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_JORNADA_720SV ((int)&platid_mask_MACH_HP_JORNADA_720SV)
#endif
extern platid_t platid_mask_MACH_HP_JORNADA_720ES;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_JORNADA_720ES ((int)&platid_mask_MACH_HP_JORNADA_720ES)
#endif
extern platid_t platid_mask_MACH_HP_JORNADA_8XX;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_JORNADA_8XX ((int)&platid_mask_MACH_HP_JORNADA_8XX)
#endif
extern platid_t platid_mask_MACH_HP_JORNADA_820;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_JORNADA_820 ((int)&platid_mask_MACH_HP_JORNADA_820)
#endif
extern platid_t platid_mask_MACH_HP_JORNADA_820JP;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HP_JORNADA_820JP ((int)&platid_mask_MACH_HP_JORNADA_820JP)
#endif
#endif /* hpcarm */
#ifdef hpcsh
extern platid_t platid_mask_MACH_HITACHI;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HITACHI ((int)&platid_mask_MACH_HITACHI)
#endif
extern platid_t platid_mask_MACH_HITACHI_PERSONA;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HITACHI_PERSONA ((int)&platid_mask_MACH_HITACHI_PERSONA)
#endif
extern platid_t platid_mask_MACH_HITACHI_PERSONA_HPW230JC;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HITACHI_PERSONA_HPW230JC ((int)&platid_mask_MACH_HITACHI_PERSONA_HPW230JC)
#endif
extern platid_t platid_mask_MACH_HITACHI_PERSONA_HPW50PAD;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HITACHI_PERSONA_HPW50PAD ((int)&platid_mask_MACH_HITACHI_PERSONA_HPW50PAD)
#endif
extern platid_t platid_mask_MACH_HITACHI_PERSONA_HPW200EC;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HITACHI_PERSONA_HPW200EC ((int)&platid_mask_MACH_HITACHI_PERSONA_HPW200EC)
#endif
extern platid_t platid_mask_MACH_HITACHI_PERSONA_HPW650PA;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define HITACHI_PERSONA_HPW650PA ((int)&platid_mask_MACH_HITACHI_PERSONA_HPW650PA)
#endif
#endif /* hpcsh */
#ifdef hpcsh
extern platid_t platid_mask_MACH_LGE;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define LGE ((int)&platid_mask_MACH_LGE)
#endif
extern platid_t platid_mask_MACH_LGE_PHENOM;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define LGE_PHENOM ((int)&platid_mask_MACH_LGE_PHENOM)
#endif
extern platid_t platid_mask_MACH_LGE_PHENOM_H220C;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define LGE_PHENOM_H220C ((int)&platid_mask_MACH_LGE_PHENOM_H220C)
#endif
#endif /* hpcsh */
#ifdef hpcmips
extern platid_t platid_mask_MACH_LASER5;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define LASER5 ((int)&platid_mask_MACH_LASER5)
#endif
extern platid_t platid_mask_MACH_LASER5_L;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define LASER5_L ((int)&platid_mask_MACH_LASER5_L)
#endif
extern platid_t platid_mask_MACH_LASER5_L_CARD;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define LASER5_L_CARD ((int)&platid_mask_MACH_LASER5_L_CARD)
#endif
extern platid_t platid_mask_MACH_LASER5_L_BOARD;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define LASER5_L_BOARD ((int)&platid_mask_MACH_LASER5_L_BOARD)
#endif
#endif /* hpcmips */
#ifdef hpcmips
extern platid_t platid_mask_MACH_AGENDA;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define AGENDA ((int)&platid_mask_MACH_AGENDA)
#endif
extern platid_t platid_mask_MACH_AGENDA_VR;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define AGENDA_VR ((int)&platid_mask_MACH_AGENDA_VR)
#endif
extern platid_t platid_mask_MACH_AGENDA_VR_VR3;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define AGENDA_VR_VR3 ((int)&platid_mask_MACH_AGENDA_VR_VR3)
#endif
#endif /* hpcmips */
#ifdef hpcarm
extern platid_t platid_mask_MACH_PSIONTEKLOGIX;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define PSIONTEKLOGIX ((int)&platid_mask_MACH_PSIONTEKLOGIX)
#endif
extern platid_t platid_mask_MACH_PSIONTEKLOGIX_NETBOOK;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define PSIONTEKLOGIX_NETBOOK ((int)&platid_mask_MACH_PSIONTEKLOGIX_NETBOOK)
#endif
extern platid_t platid_mask_MACH_PSIONTEKLOGIX_NETBOOK_PRO;
#ifdef PLATID_DEFINE_MASK_NICKNAME
#  define PSIONTEKLOGIX_NETBOOK_PRO ((int)&platid_mask_MACH_PSIONTEKLOGIX_NETBOOK_PRO)
#endif
#endif /* hpcarm */
