/*	$NetBSD: bswap.h,v 1.2 2024/08/04 08:16:25 skrll Exp $	*/

#ifndef _RISCV_BSWAP_H_
#define _RISCV_BSWAP_H_

#include <riscv/byte_swap.h>

#define __BSWAP_RENAME
#include <sys/bswap.h>

#endif /* _RISCV_BSWAP_H_ */
