/*	$NetBSD: aout_machdep.h,v 1.1 2006/09/01 21:26:18 uwe Exp $	*/

#ifndef _LANDISK_AOUT_MACHDEP_H_
#define _LANDISK_AOUT_MACHDEP_H_

#include <sh3/aout_machdep.h>

#endif /* _LANDISK_AOUT_MACHDEP_H_ */
