/*	$NetBSD: types.h,v 1.24 2021/04/01 04:42:59 simonb Exp $	*/

#ifndef _MACHINE_TYPES_H_
#define	_MACHINE_TYPES_H_

#include <m68k/types.h>

#define	__GENERIC_SOFT_INTERRUPTS_ALL_LEVELS
#define	__HAVE_LEGACY_INTRCNT
#define	__HAVE_MM_MD_READWRITE

#endif
