/*       $NetBSD: disklabel.h,v 1.2 2018/04/01 04:35:03 ryo Exp $        */

#if HAVE_NBTOOL_CONFIG_H
#include <nbinclude/arm/disklabel.h>
#else
#include <arm/disklabel.h>
#endif /* HAVE_NBTOOL_CONFIG_H */
