/* $NetBSD: bus_funcs.h,v 1.2 2011/07/09 17:22:08 dyoung Exp $ */

#ifndef _ALGOR_BUS_FUNCS_H_
#define	_ALGOR_BUS_FUNCS_H_

#include <mips/bus_space_funcs.h>
#include <mips/bus_dma_funcs.h>

#endif /* _ALGOR_BUS_FUNCS_H_ */
