/* $NetBSD: types.h,v 1.13 2021/04/01 04:35:46 simonb Exp $ */

#ifndef _NETWINDER_TYPES_H_
#define	_NETWINDER_TYPES_H_

#include <arm/arm32/types.h>

#define	__HAVE_NEW_STYLE_BUS_H
#define	__HAVE_COMPAT_NETBSD32

#endif
