/*	$NetBSD: mips32_subr.S,v 1.6 2011/07/31 15:39:29 matt Exp $	*/

#undef MIPS1
#undef MIPS3
#undef MIPS3_LOONGSON2
/* #undef MIPS32 */
#undef MIPS32R2
#undef MIPS64
#undef MIPS64R2
#undef MIPS64_SB1
#undef MIPS64_XLP
#undef MIPS64_XLR
#undef MIPS64_XLS

#include <mips/mips/mipsX_subr.S>
