/*	$NetBSD: platid_generated.h,v 1.25 2011/05/18 12:11:46 nonaka Exp $	*/

/*-
 * Copyright (c) 1999-2001
 *         Shin Takemura and PocketBSD Project. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *        This product includes software developed by the NetBSD
 *        Foundation, Inc. and its contributors.
 * 4. Neither the name of The NetBSD Foundation nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
/*
 *  Do not edit.
 *  This file is automatically generated by platid.awk.
 */
#ifdef hpcmips
#define PLATID_CPU_MIPS_NUM	1
#define PLATID_CPU_MIPS	\
  ((PLATID_CPU_MIPS_NUM << PLATID_CPU_ARCH_SHIFT))
#define PLATID_CPU_MIPS_VR_NUM	1
#define PLATID_CPU_MIPS_VR	\
  ((PLATID_CPU_MIPS_VR_NUM << PLATID_CPU_SERIES_SHIFT)| \
    PLATID_CPU_MIPS)
#define PLATID_CPU_MIPS_VR_41XX_NUM	1
#define PLATID_CPU_MIPS_VR_41XX	\
  ((PLATID_CPU_MIPS_VR_41XX_NUM << PLATID_CPU_MODEL_SHIFT)| \
    PLATID_CPU_MIPS_VR)
#define PLATID_CPU_MIPS_VR_4102_NUM	1
#define PLATID_CPU_MIPS_VR_4102	\
  ((PLATID_CPU_MIPS_VR_4102_NUM << PLATID_CPU_SUBMODEL_SHIFT)| \
    PLATID_CPU_MIPS_VR_41XX)
#define PLATID_CPU_MIPS_VR_4111_NUM	2
#define PLATID_CPU_MIPS_VR_4111	\
  ((PLATID_CPU_MIPS_VR_4111_NUM << PLATID_CPU_SUBMODEL_SHIFT)| \
    PLATID_CPU_MIPS_VR_41XX)
#define PLATID_CPU_MIPS_VR_4121_NUM	3
#define PLATID_CPU_MIPS_VR_4121	\
  ((PLATID_CPU_MIPS_VR_4121_NUM << PLATID_CPU_SUBMODEL_SHIFT)| \
    PLATID_CPU_MIPS_VR_41XX)
#define PLATID_CPU_MIPS_VR_4181_NUM	4
#define PLATID_CPU_MIPS_VR_4181	\
  ((PLATID_CPU_MIPS_VR_4181_NUM << PLATID_CPU_SUBMODEL_SHIFT)| \
    PLATID_CPU_MIPS_VR_41XX)
#define PLATID_CPU_MIPS_VR_4122_NUM	5
#define PLATID_CPU_MIPS_VR_4122	\
  ((PLATID_CPU_MIPS_VR_4122_NUM << PLATID_CPU_SUBMODEL_SHIFT)| \
    PLATID_CPU_MIPS_VR_41XX)
#define PLATID_CPU_MIPS_VR_4131_NUM	6
#define PLATID_CPU_MIPS_VR_4131	\
  ((PLATID_CPU_MIPS_VR_4131_NUM << PLATID_CPU_SUBMODEL_SHIFT)| \
    PLATID_CPU_MIPS_VR_41XX)
#define PLATID_CPU_MIPS_VR_4181A_NUM	7
#define PLATID_CPU_MIPS_VR_4181A	\
  ((PLATID_CPU_MIPS_VR_4181A_NUM << PLATID_CPU_SUBMODEL_SHIFT)| \
    PLATID_CPU_MIPS_VR_41XX)
#define PLATID_CPU_MIPS_TX_NUM	2
#define PLATID_CPU_MIPS_TX	\
  ((PLATID_CPU_MIPS_TX_NUM << PLATID_CPU_SERIES_SHIFT)| \
    PLATID_CPU_MIPS)
#define PLATID_CPU_MIPS_TX_3900_NUM	1
#define PLATID_CPU_MIPS_TX_3900	\
  ((PLATID_CPU_MIPS_TX_3900_NUM << PLATID_CPU_MODEL_SHIFT)| \
    PLATID_CPU_MIPS_TX)
#define PLATID_CPU_MIPS_TX_3911_NUM	1
#define PLATID_CPU_MIPS_TX_3911	\
  ((PLATID_CPU_MIPS_TX_3911_NUM << PLATID_CPU_SUBMODEL_SHIFT)| \
    PLATID_CPU_MIPS_TX_3900)
#define PLATID_CPU_MIPS_TX_3912_NUM	2
#define PLATID_CPU_MIPS_TX_3912	\
  ((PLATID_CPU_MIPS_TX_3912_NUM << PLATID_CPU_SUBMODEL_SHIFT)| \
    PLATID_CPU_MIPS_TX_3900)
#define PLATID_CPU_MIPS_TX_3920_NUM	2
#define PLATID_CPU_MIPS_TX_3920	\
  ((PLATID_CPU_MIPS_TX_3920_NUM << PLATID_CPU_MODEL_SHIFT)| \
    PLATID_CPU_MIPS_TX)
#define PLATID_CPU_MIPS_TX_3922_NUM	1
#define PLATID_CPU_MIPS_TX_3922	\
  ((PLATID_CPU_MIPS_TX_3922_NUM << PLATID_CPU_SUBMODEL_SHIFT)| \
    PLATID_CPU_MIPS_TX_3920)
#define PLATID_CPU_MIPS_TX_3927_NUM	2
#define PLATID_CPU_MIPS_TX_3927	\
  ((PLATID_CPU_MIPS_TX_3927_NUM << PLATID_CPU_SUBMODEL_SHIFT)| \
    PLATID_CPU_MIPS_TX_3920)
#endif /* hpcmips */
#ifdef hpcsh
#define PLATID_CPU_SH_NUM	2
#define PLATID_CPU_SH	\
  ((PLATID_CPU_SH_NUM << PLATID_CPU_ARCH_SHIFT))
#define PLATID_CPU_SH_3_NUM	1
#define PLATID_CPU_SH_3	\
  ((PLATID_CPU_SH_3_NUM << PLATID_CPU_SERIES_SHIFT)| \
    PLATID_CPU_SH)
#define PLATID_CPU_SH_3_7709_NUM	1
#define PLATID_CPU_SH_3_7709	\
  ((PLATID_CPU_SH_3_7709_NUM << PLATID_CPU_MODEL_SHIFT)| \
    PLATID_CPU_SH_3)
#define PLATID_CPU_SH_3_7709A_NUM	2
#define PLATID_CPU_SH_3_7709A	\
  ((PLATID_CPU_SH_3_7709A_NUM << PLATID_CPU_MODEL_SHIFT)| \
    PLATID_CPU_SH_3)
#define PLATID_CPU_SH_3_7707_NUM	3
#define PLATID_CPU_SH_3_7707	\
  ((PLATID_CPU_SH_3_7707_NUM << PLATID_CPU_MODEL_SHIFT)| \
    PLATID_CPU_SH_3)
#define PLATID_CPU_SH_4_NUM	2
#define PLATID_CPU_SH_4	\
  ((PLATID_CPU_SH_4_NUM << PLATID_CPU_SERIES_SHIFT)| \
    PLATID_CPU_SH)
#define PLATID_CPU_SH_4_7750_NUM	1
#define PLATID_CPU_SH_4_7750	\
  ((PLATID_CPU_SH_4_7750_NUM << PLATID_CPU_MODEL_SHIFT)| \
    PLATID_CPU_SH_4)
#endif /* hpcsh */
#ifdef hpcarm
#define PLATID_CPU_ARM_NUM	3
#define PLATID_CPU_ARM	\
  ((PLATID_CPU_ARM_NUM << PLATID_CPU_ARCH_SHIFT))
#define PLATID_CPU_ARM_STRONGARM_NUM	1
#define PLATID_CPU_ARM_STRONGARM	\
  ((PLATID_CPU_ARM_STRONGARM_NUM << PLATID_CPU_SERIES_SHIFT)| \
    PLATID_CPU_ARM)
#define PLATID_CPU_ARM_STRONGARM_SA1100_NUM	1
#define PLATID_CPU_ARM_STRONGARM_SA1100	\
  ((PLATID_CPU_ARM_STRONGARM_SA1100_NUM << PLATID_CPU_MODEL_SHIFT)| \
    PLATID_CPU_ARM_STRONGARM)
#define PLATID_CPU_ARM_STRONGARM_SA1110_NUM	2
#define PLATID_CPU_ARM_STRONGARM_SA1110	\
  ((PLATID_CPU_ARM_STRONGARM_SA1110_NUM << PLATID_CPU_MODEL_SHIFT)| \
    PLATID_CPU_ARM_STRONGARM)
#define PLATID_CPU_ARM_XSCALE_NUM	2
#define PLATID_CPU_ARM_XSCALE	\
  ((PLATID_CPU_ARM_XSCALE_NUM << PLATID_CPU_SERIES_SHIFT)| \
    PLATID_CPU_ARM)
#define PLATID_CPU_ARM_XSCALE_PXA250_NUM	1
#define PLATID_CPU_ARM_XSCALE_PXA250	\
  ((PLATID_CPU_ARM_XSCALE_PXA250_NUM << PLATID_CPU_MODEL_SHIFT)| \
    PLATID_CPU_ARM_XSCALE)
#define PLATID_CPU_ARM_XSCALE_PXA270_NUM	2
#define PLATID_CPU_ARM_XSCALE_PXA270	\
  ((PLATID_CPU_ARM_XSCALE_PXA270_NUM << PLATID_CPU_MODEL_SHIFT)| \
    PLATID_CPU_ARM_XSCALE)
#endif /* hpcarm */
#ifdef hpcmips
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_NUM	1
#define PLATID_MACH_NEC	\
  ((PLATID_MACH_NEC_NUM << PLATID_VENDOR_SHIFT))
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC_MCCS
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_MCCS_NUM	1
#define PLATID_MACH_NEC_MCCS	\
  ((PLATID_MACH_NEC_MCCS_NUM << PLATID_SERIES_SHIFT)| \
    PLATID_MACH_NEC)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC_MCCS_1X
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_MCCS_1X_NUM	1
#define PLATID_MACH_NEC_MCCS_1X	\
  ((PLATID_MACH_NEC_MCCS_1X_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_NEC_MCCS)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC_MCCS_11
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_MCCS_11_NUM	1
#define PLATID_MACH_NEC_MCCS_11	\
  ((PLATID_MACH_NEC_MCCS_11_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_NEC_MCCS_1X)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC_MCCS_12
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_MCCS_12_NUM	2
#define PLATID_MACH_NEC_MCCS_12	\
  ((PLATID_MACH_NEC_MCCS_12_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_NEC_MCCS_1X)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC_MCCS_13
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_MCCS_13_NUM	3
#define PLATID_MACH_NEC_MCCS_13	\
  ((PLATID_MACH_NEC_MCCS_13_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_NEC_MCCS_1X)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC_MCR
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_MCR_NUM	2
#define PLATID_MACH_NEC_MCR	\
  ((PLATID_MACH_NEC_MCR_NUM << PLATID_SERIES_SHIFT)| \
    PLATID_MACH_NEC)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC_MCR_3XX
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_MCR_3XX_NUM	1
#define PLATID_MACH_NEC_MCR_3XX	\
  ((PLATID_MACH_NEC_MCR_3XX_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_NEC_MCR)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC_MCR_300
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_MCR_300_NUM	1
#define PLATID_MACH_NEC_MCR_300	\
  ((PLATID_MACH_NEC_MCR_300_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_NEC_MCR_3XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC_MCR_320
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_MCR_320_NUM	2
#define PLATID_MACH_NEC_MCR_320	\
  ((PLATID_MACH_NEC_MCR_320_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_NEC_MCR_3XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC_MCR_FORDOCOMO
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_MCR_FORDOCOMO_NUM	3
#define PLATID_MACH_NEC_MCR_FORDOCOMO	\
  ((PLATID_MACH_NEC_MCR_FORDOCOMO_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_NEC_MCR_3XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC_MCR_MPRO700
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_MCR_MPRO700_NUM	4
#define PLATID_MACH_NEC_MCR_MPRO700	\
  ((PLATID_MACH_NEC_MCR_MPRO700_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_NEC_MCR_3XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC_MCR_330
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_MCR_330_NUM	5
#define PLATID_MACH_NEC_MCR_330	\
  ((PLATID_MACH_NEC_MCR_330_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_NEC_MCR_3XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC_MCR_5XX
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_MCR_5XX_NUM	2
#define PLATID_MACH_NEC_MCR_5XX	\
  ((PLATID_MACH_NEC_MCR_5XX_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_NEC_MCR)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC_MCR_500
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_MCR_500_NUM	1
#define PLATID_MACH_NEC_MCR_500	\
  ((PLATID_MACH_NEC_MCR_500_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_NEC_MCR_5XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC_MCR_510
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_MCR_510_NUM	2
#define PLATID_MACH_NEC_MCR_510	\
  ((PLATID_MACH_NEC_MCR_510_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_NEC_MCR_5XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC_MCR_520
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_MCR_520_NUM	3
#define PLATID_MACH_NEC_MCR_520	\
  ((PLATID_MACH_NEC_MCR_520_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_NEC_MCR_5XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC_MCR_520A
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_MCR_520A_NUM	4
#define PLATID_MACH_NEC_MCR_520A	\
  ((PLATID_MACH_NEC_MCR_520A_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_NEC_MCR_5XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC_MCR_500A
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_MCR_500A_NUM	5
#define PLATID_MACH_NEC_MCR_500A	\
  ((PLATID_MACH_NEC_MCR_500A_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_NEC_MCR_5XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC_MCR_530
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_MCR_530_NUM	6
#define PLATID_MACH_NEC_MCR_530	\
  ((PLATID_MACH_NEC_MCR_530_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_NEC_MCR_5XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC_MCR_430
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_MCR_430_NUM	7
#define PLATID_MACH_NEC_MCR_430	\
  ((PLATID_MACH_NEC_MCR_430_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_NEC_MCR_5XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC_MCR_530A
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_MCR_530A_NUM	8
#define PLATID_MACH_NEC_MCR_530A	\
  ((PLATID_MACH_NEC_MCR_530A_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_NEC_MCR_5XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC_MCR_SIGMARION
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_MCR_SIGMARION_NUM	9
#define PLATID_MACH_NEC_MCR_SIGMARION	\
  ((PLATID_MACH_NEC_MCR_SIGMARION_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_NEC_MCR_5XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC_MCR_550
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_MCR_550_NUM	10
#define PLATID_MACH_NEC_MCR_550	\
  ((PLATID_MACH_NEC_MCR_550_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_NEC_MCR_5XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC_MCR_450
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_MCR_450_NUM	11
#define PLATID_MACH_NEC_MCR_450	\
  ((PLATID_MACH_NEC_MCR_450_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_NEC_MCR_5XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC_MCR_SIGMARION2
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_MCR_SIGMARION2_NUM	12
#define PLATID_MACH_NEC_MCR_SIGMARION2	\
  ((PLATID_MACH_NEC_MCR_SIGMARION2_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_NEC_MCR_5XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC_MCR_7XX
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_MCR_7XX_NUM	3
#define PLATID_MACH_NEC_MCR_7XX	\
  ((PLATID_MACH_NEC_MCR_7XX_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_NEC_MCR)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC_MCR_700
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_MCR_700_NUM	1
#define PLATID_MACH_NEC_MCR_700	\
  ((PLATID_MACH_NEC_MCR_700_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_NEC_MCR_7XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC_MCR_700A
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_MCR_700A_NUM	2
#define PLATID_MACH_NEC_MCR_700A	\
  ((PLATID_MACH_NEC_MCR_700A_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_NEC_MCR_7XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC_MCR_730
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_MCR_730_NUM	3
#define PLATID_MACH_NEC_MCR_730	\
  ((PLATID_MACH_NEC_MCR_730_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_NEC_MCR_7XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_NEC_MCR_730A
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_NEC_MCR_730A_NUM	4
#define PLATID_MACH_NEC_MCR_730A	\
  ((PLATID_MACH_NEC_MCR_730A_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_NEC_MCR_7XX)
#endif /* hpcmips */
#ifdef hpcmips
#ifndef SPEC_PLATFORM
#define SPEC_MACH_EVEREX
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_EVEREX_NUM	2
#define PLATID_MACH_EVEREX	\
  ((PLATID_MACH_EVEREX_NUM << PLATID_VENDOR_SHIFT))
#ifndef SPEC_PLATFORM
#define SPEC_MACH_EVEREX_FREESTYLE
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_EVEREX_FREESTYLE_NUM	1
#define PLATID_MACH_EVEREX_FREESTYLE	\
  ((PLATID_MACH_EVEREX_FREESTYLE_NUM << PLATID_SERIES_SHIFT)| \
    PLATID_MACH_EVEREX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_EVEREX_FREESTYLE_AXX
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_EVEREX_FREESTYLE_AXX_NUM	1
#define PLATID_MACH_EVEREX_FREESTYLE_AXX	\
  ((PLATID_MACH_EVEREX_FREESTYLE_AXX_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_EVEREX_FREESTYLE)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_EVEREX_FREESTYLE_A10
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_EVEREX_FREESTYLE_A10_NUM	1
#define PLATID_MACH_EVEREX_FREESTYLE_A10	\
  ((PLATID_MACH_EVEREX_FREESTYLE_A10_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_EVEREX_FREESTYLE_AXX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_EVEREX_FREESTYLE_A15
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_EVEREX_FREESTYLE_A15_NUM	2
#define PLATID_MACH_EVEREX_FREESTYLE_A15	\
  ((PLATID_MACH_EVEREX_FREESTYLE_A15_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_EVEREX_FREESTYLE_AXX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_EVEREX_FREESTYLE_A20
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_EVEREX_FREESTYLE_A20_NUM	3
#define PLATID_MACH_EVEREX_FREESTYLE_A20	\
  ((PLATID_MACH_EVEREX_FREESTYLE_A20_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_EVEREX_FREESTYLE_AXX)
#endif /* hpcmips */
#ifndef SPEC_PLATFORM
#define SPEC_MACH_CASIO
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_CASIO_NUM	3
#define PLATID_MACH_CASIO	\
  ((PLATID_MACH_CASIO_NUM << PLATID_VENDOR_SHIFT))
#ifdef hpcmips
#ifndef SPEC_PLATFORM
#define SPEC_MACH_CASIO_CASSIOPEIAE
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_CASIO_CASSIOPEIAE_NUM	1
#define PLATID_MACH_CASIO_CASSIOPEIAE	\
  ((PLATID_MACH_CASIO_CASSIOPEIAE_NUM << PLATID_SERIES_SHIFT)| \
    PLATID_MACH_CASIO)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_CASIO_CASSIOPEIAE_EXX
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_CASIO_CASSIOPEIAE_EXX_NUM	1
#define PLATID_MACH_CASIO_CASSIOPEIAE_EXX	\
  ((PLATID_MACH_CASIO_CASSIOPEIAE_EXX_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_CASIO_CASSIOPEIAE)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_CASIO_CASSIOPEIAE_E10
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_CASIO_CASSIOPEIAE_E10_NUM	1
#define PLATID_MACH_CASIO_CASSIOPEIAE_E10	\
  ((PLATID_MACH_CASIO_CASSIOPEIAE_E10_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_CASIO_CASSIOPEIAE_EXX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_CASIO_CASSIOPEIAE_E11
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_CASIO_CASSIOPEIAE_E11_NUM	2
#define PLATID_MACH_CASIO_CASSIOPEIAE_E11	\
  ((PLATID_MACH_CASIO_CASSIOPEIAE_E11_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_CASIO_CASSIOPEIAE_EXX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_CASIO_CASSIOPEIAE_E15
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_CASIO_CASSIOPEIAE_E15_NUM	3
#define PLATID_MACH_CASIO_CASSIOPEIAE_E15	\
  ((PLATID_MACH_CASIO_CASSIOPEIAE_E15_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_CASIO_CASSIOPEIAE_EXX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_CASIO_CASSIOPEIAE_E55
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_CASIO_CASSIOPEIAE_E55_NUM	4
#define PLATID_MACH_CASIO_CASSIOPEIAE_E55	\
  ((PLATID_MACH_CASIO_CASSIOPEIAE_E55_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_CASIO_CASSIOPEIAE_EXX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_CASIO_CASSIOPEIAE_FORDOCOMO
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_CASIO_CASSIOPEIAE_FORDOCOMO_NUM	5
#define PLATID_MACH_CASIO_CASSIOPEIAE_FORDOCOMO	\
  ((PLATID_MACH_CASIO_CASSIOPEIAE_FORDOCOMO_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_CASIO_CASSIOPEIAE_EXX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_CASIO_CASSIOPEIAE_E65
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_CASIO_CASSIOPEIAE_E65_NUM	6
#define PLATID_MACH_CASIO_CASSIOPEIAE_E65	\
  ((PLATID_MACH_CASIO_CASSIOPEIAE_E65_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_CASIO_CASSIOPEIAE_EXX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_CASIO_CASSIOPEIAE_EXXX
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_CASIO_CASSIOPEIAE_EXXX_NUM	2
#define PLATID_MACH_CASIO_CASSIOPEIAE_EXXX	\
  ((PLATID_MACH_CASIO_CASSIOPEIAE_EXXX_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_CASIO_CASSIOPEIAE)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_CASIO_CASSIOPEIAE_E100
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_CASIO_CASSIOPEIAE_E100_NUM	1
#define PLATID_MACH_CASIO_CASSIOPEIAE_E100	\
  ((PLATID_MACH_CASIO_CASSIOPEIAE_E100_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_CASIO_CASSIOPEIAE_EXXX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_CASIO_CASSIOPEIAE_E105
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_CASIO_CASSIOPEIAE_E105_NUM	2
#define PLATID_MACH_CASIO_CASSIOPEIAE_E105	\
  ((PLATID_MACH_CASIO_CASSIOPEIAE_E105_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_CASIO_CASSIOPEIAE_EXXX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_CASIO_CASSIOPEIAE_E500
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_CASIO_CASSIOPEIAE_E500_NUM	3
#define PLATID_MACH_CASIO_CASSIOPEIAE_E500	\
  ((PLATID_MACH_CASIO_CASSIOPEIAE_E500_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_CASIO_CASSIOPEIAE_EXXX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_CASIO_CASSIOPEIAE_E507
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_CASIO_CASSIOPEIAE_E507_NUM	4
#define PLATID_MACH_CASIO_CASSIOPEIAE_E507	\
  ((PLATID_MACH_CASIO_CASSIOPEIAE_E507_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_CASIO_CASSIOPEIAE_EXXX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_CASIO_POCKETPOSTPET
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_CASIO_POCKETPOSTPET_NUM	2
#define PLATID_MACH_CASIO_POCKETPOSTPET	\
  ((PLATID_MACH_CASIO_POCKETPOSTPET_NUM << PLATID_SERIES_SHIFT)| \
    PLATID_MACH_CASIO)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_CASIO_POCKETPOSTPET_POCKETPOSTPET
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_CASIO_POCKETPOSTPET_POCKETPOSTPET_NUM	1
#define PLATID_MACH_CASIO_POCKETPOSTPET_POCKETPOSTPET	\
  ((PLATID_MACH_CASIO_POCKETPOSTPET_POCKETPOSTPET_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_CASIO_POCKETPOSTPET)
#endif /* hpcmips */
#ifdef hpcsh
#ifndef SPEC_PLATFORM
#define SPEC_MACH_CASIO_CASSIOPEIAA
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_CASIO_CASSIOPEIAA_NUM	3
#define PLATID_MACH_CASIO_CASSIOPEIAA	\
  ((PLATID_MACH_CASIO_CASSIOPEIAA_NUM << PLATID_SERIES_SHIFT)| \
    PLATID_MACH_CASIO)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_CASIO_CASSIOPEIAA_AXX
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_CASIO_CASSIOPEIAA_AXX_NUM	1
#define PLATID_MACH_CASIO_CASSIOPEIAA_AXX	\
  ((PLATID_MACH_CASIO_CASSIOPEIAA_AXX_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_CASIO_CASSIOPEIAA)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_CASIO_CASSIOPEIAA_A55V
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_CASIO_CASSIOPEIAA_A55V_NUM	1
#define PLATID_MACH_CASIO_CASSIOPEIAA_A55V	\
  ((PLATID_MACH_CASIO_CASSIOPEIAA_A55V_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_CASIO_CASSIOPEIAA_AXX)
#endif /* hpcsh */
#if defined(hpcmips) || defined(hpcarm)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_SHARP
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_SHARP_NUM	4
#define PLATID_MACH_SHARP	\
  ((PLATID_MACH_SHARP_NUM << PLATID_VENDOR_SHIFT))
#if defined(hpcmips)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_SHARP_TRIPAD
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_SHARP_TRIPAD_NUM	1
#define PLATID_MACH_SHARP_TRIPAD	\
  ((PLATID_MACH_SHARP_TRIPAD_NUM << PLATID_SERIES_SHIFT)| \
    PLATID_MACH_SHARP)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_SHARP_TRIPAD_PV
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_SHARP_TRIPAD_PV_NUM	1
#define PLATID_MACH_SHARP_TRIPAD_PV	\
  ((PLATID_MACH_SHARP_TRIPAD_PV_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_SHARP_TRIPAD)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_SHARP_TRIPAD_PV6000
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_SHARP_TRIPAD_PV6000_NUM	1
#define PLATID_MACH_SHARP_TRIPAD_PV6000	\
  ((PLATID_MACH_SHARP_TRIPAD_PV6000_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_SHARP_TRIPAD_PV)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_SHARP_TELIOS
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_SHARP_TELIOS_NUM	2
#define PLATID_MACH_SHARP_TELIOS	\
  ((PLATID_MACH_SHARP_TELIOS_NUM << PLATID_SERIES_SHIFT)| \
    PLATID_MACH_SHARP)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_SHARP_TELIOS_HCAJ
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_SHARP_TELIOS_HCAJ_NUM	1
#define PLATID_MACH_SHARP_TELIOS_HCAJ	\
  ((PLATID_MACH_SHARP_TELIOS_HCAJ_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_SHARP_TELIOS)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_SHARP_TELIOS_HCAJ1_JP
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_SHARP_TELIOS_HCAJ1_JP_NUM	1
#define PLATID_MACH_SHARP_TELIOS_HCAJ1_JP	\
  ((PLATID_MACH_SHARP_TELIOS_HCAJ1_JP_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_SHARP_TELIOS_HCAJ)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_SHARP_TELIOS_HCAJ2_JP
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_SHARP_TELIOS_HCAJ2_JP_NUM	2
#define PLATID_MACH_SHARP_TELIOS_HCAJ2_JP	\
  ((PLATID_MACH_SHARP_TELIOS_HCAJ2_JP_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_SHARP_TELIOS_HCAJ)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_SHARP_TELIOS_HCAJ3_JP
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_SHARP_TELIOS_HCAJ3_JP_NUM	3
#define PLATID_MACH_SHARP_TELIOS_HCAJ3_JP	\
  ((PLATID_MACH_SHARP_TELIOS_HCAJ3_JP_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_SHARP_TELIOS_HCAJ)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_SHARP_TELIOS_HCVJ
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_SHARP_TELIOS_HCVJ_NUM	2
#define PLATID_MACH_SHARP_TELIOS_HCVJ	\
  ((PLATID_MACH_SHARP_TELIOS_HCVJ_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_SHARP_TELIOS)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_SHARP_TELIOS_HCVJ1C_JP
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_SHARP_TELIOS_HCVJ1C_JP_NUM	1
#define PLATID_MACH_SHARP_TELIOS_HCVJ1C_JP	\
  ((PLATID_MACH_SHARP_TELIOS_HCVJ1C_JP_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_SHARP_TELIOS_HCVJ)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_SHARP_MOBILON
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_SHARP_MOBILON_NUM	3
#define PLATID_MACH_SHARP_MOBILON	\
  ((PLATID_MACH_SHARP_MOBILON_NUM << PLATID_SERIES_SHIFT)| \
    PLATID_MACH_SHARP)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_SHARP_MOBILON_HC
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_SHARP_MOBILON_HC_NUM	1
#define PLATID_MACH_SHARP_MOBILON_HC	\
  ((PLATID_MACH_SHARP_MOBILON_HC_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_SHARP_MOBILON)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_SHARP_MOBILON_HC4100
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_SHARP_MOBILON_HC4100_NUM	1
#define PLATID_MACH_SHARP_MOBILON_HC4100	\
  ((PLATID_MACH_SHARP_MOBILON_HC4100_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_SHARP_MOBILON_HC)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_SHARP_MOBILON_HC4500
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_SHARP_MOBILON_HC4500_NUM	2
#define PLATID_MACH_SHARP_MOBILON_HC4500	\
  ((PLATID_MACH_SHARP_MOBILON_HC4500_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_SHARP_MOBILON_HC)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_SHARP_MOBILON_HC1200
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_SHARP_MOBILON_HC1200_NUM	3
#define PLATID_MACH_SHARP_MOBILON_HC1200	\
  ((PLATID_MACH_SHARP_MOBILON_HC1200_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_SHARP_MOBILON_HC)
#endif /* hpcmips */
#if defined(hpcarm)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_SHARP_WZERO3
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_SHARP_WZERO3_NUM	4
#define PLATID_MACH_SHARP_WZERO3	\
  ((PLATID_MACH_SHARP_WZERO3_NUM << PLATID_SERIES_SHIFT)| \
    PLATID_MACH_SHARP)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_SHARP_WZERO3_WS003SH
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_SHARP_WZERO3_WS003SH_NUM	1
#define PLATID_MACH_SHARP_WZERO3_WS003SH	\
  ((PLATID_MACH_SHARP_WZERO3_WS003SH_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_SHARP_WZERO3)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_SHARP_WZERO3_WS004SH
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_SHARP_WZERO3_WS004SH_NUM	2
#define PLATID_MACH_SHARP_WZERO3_WS004SH	\
  ((PLATID_MACH_SHARP_WZERO3_WS004SH_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_SHARP_WZERO3)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_SHARP_WZERO3_WS007SH
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_SHARP_WZERO3_WS007SH_NUM	3
#define PLATID_MACH_SHARP_WZERO3_WS007SH	\
  ((PLATID_MACH_SHARP_WZERO3_WS007SH_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_SHARP_WZERO3)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_SHARP_WZERO3_WS011SH
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_SHARP_WZERO3_WS011SH_NUM	4
#define PLATID_MACH_SHARP_WZERO3_WS011SH	\
  ((PLATID_MACH_SHARP_WZERO3_WS011SH_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_SHARP_WZERO3)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_SHARP_WZERO3_WS020SH
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_SHARP_WZERO3_WS020SH_NUM	5
#define PLATID_MACH_SHARP_WZERO3_WS020SH	\
  ((PLATID_MACH_SHARP_WZERO3_WS020SH_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_SHARP_WZERO3)
#endif /* hpcarm */
#endif /* hpcmips || hpcarm */
#ifdef hpcmips
#ifndef SPEC_PLATFORM
#define SPEC_MACH_FUJITSU
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_FUJITSU_NUM	5
#define PLATID_MACH_FUJITSU	\
  ((PLATID_MACH_FUJITSU_NUM << PLATID_VENDOR_SHIFT))
#ifndef SPEC_PLATFORM
#define SPEC_MACH_FUJITSU_INTERTOP
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_FUJITSU_INTERTOP_NUM	1
#define PLATID_MACH_FUJITSU_INTERTOP	\
  ((PLATID_MACH_FUJITSU_INTERTOP_NUM << PLATID_SERIES_SHIFT)| \
    PLATID_MACH_FUJITSU)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_FUJITSU_INTERTOP_ITXXX
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_FUJITSU_INTERTOP_ITXXX_NUM	1
#define PLATID_MACH_FUJITSU_INTERTOP_ITXXX	\
  ((PLATID_MACH_FUJITSU_INTERTOP_ITXXX_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_FUJITSU_INTERTOP)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_FUJITSU_INTERTOP_IT300
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_FUJITSU_INTERTOP_IT300_NUM	1
#define PLATID_MACH_FUJITSU_INTERTOP_IT300	\
  ((PLATID_MACH_FUJITSU_INTERTOP_IT300_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_FUJITSU_INTERTOP_ITXXX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_FUJITSU_INTERTOP_IT310
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_FUJITSU_INTERTOP_IT310_NUM	2
#define PLATID_MACH_FUJITSU_INTERTOP_IT310	\
  ((PLATID_MACH_FUJITSU_INTERTOP_IT310_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_FUJITSU_INTERTOP_ITXXX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_FUJITSU_PENCENTRA
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_FUJITSU_PENCENTRA_NUM	2
#define PLATID_MACH_FUJITSU_PENCENTRA	\
  ((PLATID_MACH_FUJITSU_PENCENTRA_NUM << PLATID_SERIES_SHIFT)| \
    PLATID_MACH_FUJITSU)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_FUJITSU_PENCENTRA_130
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_FUJITSU_PENCENTRA_130_NUM	1
#define PLATID_MACH_FUJITSU_PENCENTRA_130	\
  ((PLATID_MACH_FUJITSU_PENCENTRA_130_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_FUJITSU_PENCENTRA)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_FUJITSU_PENCENTRA_130TM
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_FUJITSU_PENCENTRA_130TM_NUM	1
#define PLATID_MACH_FUJITSU_PENCENTRA_130TM	\
  ((PLATID_MACH_FUJITSU_PENCENTRA_130TM_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_FUJITSU_PENCENTRA_130)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_FUJITSU_PENCENTRA_130RF
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_FUJITSU_PENCENTRA_130RF_NUM	2
#define PLATID_MACH_FUJITSU_PENCENTRA_130RF	\
  ((PLATID_MACH_FUJITSU_PENCENTRA_130RF_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_FUJITSU_PENCENTRA_130)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_FUJITSU_PENCENTRA_200
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_FUJITSU_PENCENTRA_200_NUM	2
#define PLATID_MACH_FUJITSU_PENCENTRA_200	\
  ((PLATID_MACH_FUJITSU_PENCENTRA_200_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_FUJITSU_PENCENTRA)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_FUJITSU_PENCENTRA_200CTM
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_FUJITSU_PENCENTRA_200CTM_NUM	1
#define PLATID_MACH_FUJITSU_PENCENTRA_200CTM	\
  ((PLATID_MACH_FUJITSU_PENCENTRA_200CTM_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_FUJITSU_PENCENTRA_200)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_FUJITSU_PENCENTRA_200CRF
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_FUJITSU_PENCENTRA_200CRF_NUM	2
#define PLATID_MACH_FUJITSU_PENCENTRA_200CRF	\
  ((PLATID_MACH_FUJITSU_PENCENTRA_200CRF_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_FUJITSU_PENCENTRA_200)
#endif /* hpcmips */
#ifdef hpcmips
#ifndef SPEC_PLATFORM
#define SPEC_MACH_PHILIPS
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_PHILIPS_NUM	6
#define PLATID_MACH_PHILIPS	\
  ((PLATID_MACH_PHILIPS_NUM << PLATID_VENDOR_SHIFT))
#ifndef SPEC_PLATFORM
#define SPEC_MACH_PHILIPS_NINO
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_PHILIPS_NINO_NUM	1
#define PLATID_MACH_PHILIPS_NINO	\
  ((PLATID_MACH_PHILIPS_NINO_NUM << PLATID_SERIES_SHIFT)| \
    PLATID_MACH_PHILIPS)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_PHILIPS_NINO_3XX
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_PHILIPS_NINO_3XX_NUM	1
#define PLATID_MACH_PHILIPS_NINO_3XX	\
  ((PLATID_MACH_PHILIPS_NINO_3XX_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_PHILIPS_NINO)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_PHILIPS_NINO_312
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_PHILIPS_NINO_312_NUM	1
#define PLATID_MACH_PHILIPS_NINO_312	\
  ((PLATID_MACH_PHILIPS_NINO_312_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_PHILIPS_NINO_3XX)
#endif /* hpcmips */
#ifndef SPEC_PLATFORM
#define SPEC_MACH_COMPAQ
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_COMPAQ_NUM	7
#define PLATID_MACH_COMPAQ	\
  ((PLATID_MACH_COMPAQ_NUM << PLATID_VENDOR_SHIFT))
#ifdef hpcmips
#ifndef SPEC_PLATFORM
#define SPEC_MACH_COMPAQ_C
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_COMPAQ_C_NUM	1
#define PLATID_MACH_COMPAQ_C	\
  ((PLATID_MACH_COMPAQ_C_NUM << PLATID_SERIES_SHIFT)| \
    PLATID_MACH_COMPAQ)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_COMPAQ_C_8XX
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_COMPAQ_C_8XX_NUM	1
#define PLATID_MACH_COMPAQ_C_8XX	\
  ((PLATID_MACH_COMPAQ_C_8XX_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_COMPAQ_C)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_COMPAQ_C_810
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_COMPAQ_C_810_NUM	1
#define PLATID_MACH_COMPAQ_C_810	\
  ((PLATID_MACH_COMPAQ_C_810_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_COMPAQ_C_8XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_COMPAQ_C_201X
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_COMPAQ_C_201X_NUM	2
#define PLATID_MACH_COMPAQ_C_201X	\
  ((PLATID_MACH_COMPAQ_C_201X_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_COMPAQ_C)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_COMPAQ_C_2010
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_COMPAQ_C_2010_NUM	1
#define PLATID_MACH_COMPAQ_C_2010	\
  ((PLATID_MACH_COMPAQ_C_2010_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_COMPAQ_C_201X)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_COMPAQ_C_2015
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_COMPAQ_C_2015_NUM	2
#define PLATID_MACH_COMPAQ_C_2015	\
  ((PLATID_MACH_COMPAQ_C_2015_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_COMPAQ_C_201X)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_COMPAQ_AERO
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_COMPAQ_AERO_NUM	2
#define PLATID_MACH_COMPAQ_AERO	\
  ((PLATID_MACH_COMPAQ_AERO_NUM << PLATID_SERIES_SHIFT)| \
    PLATID_MACH_COMPAQ)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_COMPAQ_AERO_15XX
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_COMPAQ_AERO_15XX_NUM	1
#define PLATID_MACH_COMPAQ_AERO_15XX	\
  ((PLATID_MACH_COMPAQ_AERO_15XX_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_COMPAQ_AERO)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_COMPAQ_AERO_1530
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_COMPAQ_AERO_1530_NUM	1
#define PLATID_MACH_COMPAQ_AERO_1530	\
  ((PLATID_MACH_COMPAQ_AERO_1530_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_COMPAQ_AERO_15XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_COMPAQ_AERO_21XX
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_COMPAQ_AERO_21XX_NUM	2
#define PLATID_MACH_COMPAQ_AERO_21XX	\
  ((PLATID_MACH_COMPAQ_AERO_21XX_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_COMPAQ_AERO)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_COMPAQ_AERO_2110
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_COMPAQ_AERO_2110_NUM	1
#define PLATID_MACH_COMPAQ_AERO_2110	\
  ((PLATID_MACH_COMPAQ_AERO_2110_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_COMPAQ_AERO_21XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_COMPAQ_AERO_2130
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_COMPAQ_AERO_2130_NUM	2
#define PLATID_MACH_COMPAQ_AERO_2130	\
  ((PLATID_MACH_COMPAQ_AERO_2130_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_COMPAQ_AERO_21XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_COMPAQ_AERO_2140
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_COMPAQ_AERO_2140_NUM	3
#define PLATID_MACH_COMPAQ_AERO_2140	\
  ((PLATID_MACH_COMPAQ_AERO_2140_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_COMPAQ_AERO_21XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_COMPAQ_PRESARIO
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_COMPAQ_PRESARIO_NUM	3
#define PLATID_MACH_COMPAQ_PRESARIO	\
  ((PLATID_MACH_COMPAQ_PRESARIO_NUM << PLATID_SERIES_SHIFT)| \
    PLATID_MACH_COMPAQ)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_COMPAQ_PRESARIO_21X
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_COMPAQ_PRESARIO_21X_NUM	1
#define PLATID_MACH_COMPAQ_PRESARIO_21X	\
  ((PLATID_MACH_COMPAQ_PRESARIO_21X_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_COMPAQ_PRESARIO)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_COMPAQ_PRESARIO_213
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_COMPAQ_PRESARIO_213_NUM	1
#define PLATID_MACH_COMPAQ_PRESARIO_213	\
  ((PLATID_MACH_COMPAQ_PRESARIO_213_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_COMPAQ_PRESARIO_21X)
#endif /* hpcmips */
#ifdef hpcarm
#ifndef SPEC_PLATFORM
#define SPEC_MACH_COMPAQ_IPAQ
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_COMPAQ_IPAQ_NUM	4
#define PLATID_MACH_COMPAQ_IPAQ	\
  ((PLATID_MACH_COMPAQ_IPAQ_NUM << PLATID_SERIES_SHIFT)| \
    PLATID_MACH_COMPAQ)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_COMPAQ_IPAQ_H31XX
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_COMPAQ_IPAQ_H31XX_NUM	1
#define PLATID_MACH_COMPAQ_IPAQ_H31XX	\
  ((PLATID_MACH_COMPAQ_IPAQ_H31XX_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_COMPAQ_IPAQ)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_COMPAQ_IPAQ_H3100
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_COMPAQ_IPAQ_H3100_NUM	1
#define PLATID_MACH_COMPAQ_IPAQ_H3100	\
  ((PLATID_MACH_COMPAQ_IPAQ_H3100_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_COMPAQ_IPAQ_H31XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_COMPAQ_IPAQ_H36XX
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_COMPAQ_IPAQ_H36XX_NUM	2
#define PLATID_MACH_COMPAQ_IPAQ_H36XX	\
  ((PLATID_MACH_COMPAQ_IPAQ_H36XX_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_COMPAQ_IPAQ)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_COMPAQ_IPAQ_H3600
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_COMPAQ_IPAQ_H3600_NUM	1
#define PLATID_MACH_COMPAQ_IPAQ_H3600	\
  ((PLATID_MACH_COMPAQ_IPAQ_H3600_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_COMPAQ_IPAQ_H36XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_COMPAQ_IPAQ_H3660
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_COMPAQ_IPAQ_H3660_NUM	2
#define PLATID_MACH_COMPAQ_IPAQ_H3660	\
  ((PLATID_MACH_COMPAQ_IPAQ_H3660_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_COMPAQ_IPAQ_H36XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_COMPAQ_IPAQ_H39XX
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_COMPAQ_IPAQ_H39XX_NUM	3
#define PLATID_MACH_COMPAQ_IPAQ_H39XX	\
  ((PLATID_MACH_COMPAQ_IPAQ_H39XX_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_COMPAQ_IPAQ)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_COMPAQ_IPAQ_H3900
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_COMPAQ_IPAQ_H3900_NUM	1
#define PLATID_MACH_COMPAQ_IPAQ_H3900	\
  ((PLATID_MACH_COMPAQ_IPAQ_H3900_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_COMPAQ_IPAQ_H39XX)
#endif /* hpcarm */
#ifdef hpcsh
#ifndef SPEC_PLATFORM
#define SPEC_MACH_COMPAQ_AERO
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_COMPAQ_AERO_NUM	5
#define PLATID_MACH_COMPAQ_AERO	\
  ((PLATID_MACH_COMPAQ_AERO_NUM << PLATID_SERIES_SHIFT)| \
    PLATID_MACH_COMPAQ)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_COMPAQ_AERO_8000
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_COMPAQ_AERO_8000_NUM	1
#define PLATID_MACH_COMPAQ_AERO_8000	\
  ((PLATID_MACH_COMPAQ_AERO_8000_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_COMPAQ_AERO)
#endif /* hpcsh */
#ifdef hpcmips
#ifndef SPEC_PLATFORM
#define SPEC_MACH_VICTOR
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_VICTOR_NUM	8
#define PLATID_MACH_VICTOR	\
  ((PLATID_MACH_VICTOR_NUM << PLATID_VENDOR_SHIFT))
#ifndef SPEC_PLATFORM
#define SPEC_MACH_VICTOR_INTERLINK
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_VICTOR_INTERLINK_NUM	1
#define PLATID_MACH_VICTOR_INTERLINK	\
  ((PLATID_MACH_VICTOR_INTERLINK_NUM << PLATID_SERIES_SHIFT)| \
    PLATID_MACH_VICTOR)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_VICTOR_INTERLINK_MP
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_VICTOR_INTERLINK_MP_NUM	1
#define PLATID_MACH_VICTOR_INTERLINK_MP	\
  ((PLATID_MACH_VICTOR_INTERLINK_MP_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_VICTOR_INTERLINK)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_VICTOR_INTERLINK_MPC101
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_VICTOR_INTERLINK_MPC101_NUM	1
#define PLATID_MACH_VICTOR_INTERLINK_MPC101	\
  ((PLATID_MACH_VICTOR_INTERLINK_MPC101_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_VICTOR_INTERLINK_MP)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_VICTOR_INTERLINK_MPC303
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_VICTOR_INTERLINK_MPC303_NUM	2
#define PLATID_MACH_VICTOR_INTERLINK_MPC303	\
  ((PLATID_MACH_VICTOR_INTERLINK_MPC303_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_VICTOR_INTERLINK_MP)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_VICTOR_INTERLINK_MPC304
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_VICTOR_INTERLINK_MPC304_NUM	3
#define PLATID_MACH_VICTOR_INTERLINK_MPC304	\
  ((PLATID_MACH_VICTOR_INTERLINK_MPC304_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_VICTOR_INTERLINK_MP)
#endif /* hpcmips */
#ifdef hpcmips
#ifndef SPEC_PLATFORM
#define SPEC_MACH_IBM
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_IBM_NUM	9
#define PLATID_MACH_IBM	\
  ((PLATID_MACH_IBM_NUM << PLATID_VENDOR_SHIFT))
#ifndef SPEC_PLATFORM
#define SPEC_MACH_IBM_WORKPAD
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_IBM_WORKPAD_NUM	1
#define PLATID_MACH_IBM_WORKPAD	\
  ((PLATID_MACH_IBM_WORKPAD_NUM << PLATID_SERIES_SHIFT)| \
    PLATID_MACH_IBM)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_IBM_WORKPAD_Z50
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_IBM_WORKPAD_Z50_NUM	1
#define PLATID_MACH_IBM_WORKPAD_Z50	\
  ((PLATID_MACH_IBM_WORKPAD_Z50_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_IBM_WORKPAD)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_IBM_WORKPAD_26011AU
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_IBM_WORKPAD_26011AU_NUM	1
#define PLATID_MACH_IBM_WORKPAD_26011AU	\
  ((PLATID_MACH_IBM_WORKPAD_26011AU_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_IBM_WORKPAD_Z50)
#endif /* hpcmips */
#ifdef hpcmips
#ifndef SPEC_PLATFORM
#define SPEC_MACH_VADEM
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_VADEM_NUM	10
#define PLATID_MACH_VADEM	\
  ((PLATID_MACH_VADEM_NUM << PLATID_VENDOR_SHIFT))
#ifndef SPEC_PLATFORM
#define SPEC_MACH_VADEM_CLIO
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_VADEM_CLIO_NUM	1
#define PLATID_MACH_VADEM_CLIO	\
  ((PLATID_MACH_VADEM_CLIO_NUM << PLATID_SERIES_SHIFT)| \
    PLATID_MACH_VADEM)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_VADEM_CLIO_C
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_VADEM_CLIO_C_NUM	1
#define PLATID_MACH_VADEM_CLIO_C	\
  ((PLATID_MACH_VADEM_CLIO_C_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_VADEM_CLIO)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_VADEM_CLIO_C1000
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_VADEM_CLIO_C1000_NUM	1
#define PLATID_MACH_VADEM_CLIO_C1000	\
  ((PLATID_MACH_VADEM_CLIO_C1000_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_VADEM_CLIO_C)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_VADEM_CLIO_C1050
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_VADEM_CLIO_C1050_NUM	2
#define PLATID_MACH_VADEM_CLIO_C1050	\
  ((PLATID_MACH_VADEM_CLIO_C1050_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_VADEM_CLIO_C)
#endif /* hpcmips */
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_NUM	11
#define PLATID_MACH_HP	\
  ((PLATID_MACH_HP_NUM << PLATID_VENDOR_SHIFT))
#ifdef hpcsh
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_LX
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_LX_NUM	1
#define PLATID_MACH_HP_LX	\
  ((PLATID_MACH_HP_LX_NUM << PLATID_SERIES_SHIFT)| \
    PLATID_MACH_HP)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_LX_620
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_LX_620_NUM	1
#define PLATID_MACH_HP_LX_620	\
  ((PLATID_MACH_HP_LX_620_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_HP_LX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_LX_620JP
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_LX_620JP_NUM	2
#define PLATID_MACH_HP_LX_620JP	\
  ((PLATID_MACH_HP_LX_620JP_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_HP_LX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_LX_360
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_LX_360_NUM	3
#define PLATID_MACH_HP_LX_360	\
  ((PLATID_MACH_HP_LX_360_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_HP_LX)
#endif /* hpcsh */
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_JORNADA
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_JORNADA_NUM	2
#define PLATID_MACH_HP_JORNADA	\
  ((PLATID_MACH_HP_JORNADA_NUM << PLATID_SERIES_SHIFT)| \
    PLATID_MACH_HP)
#ifdef hpcsh
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_JORNADA_6XX
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_JORNADA_6XX_NUM	1
#define PLATID_MACH_HP_JORNADA_6XX	\
  ((PLATID_MACH_HP_JORNADA_6XX_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_HP_JORNADA)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_JORNADA_680
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_JORNADA_680_NUM	1
#define PLATID_MACH_HP_JORNADA_680	\
  ((PLATID_MACH_HP_JORNADA_680_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_HP_JORNADA_6XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_JORNADA_680JP
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_JORNADA_680JP_NUM	2
#define PLATID_MACH_HP_JORNADA_680JP	\
  ((PLATID_MACH_HP_JORNADA_680JP_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_HP_JORNADA_6XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_JORNADA_680EU
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_JORNADA_680EU_NUM	3
#define PLATID_MACH_HP_JORNADA_680EU	\
  ((PLATID_MACH_HP_JORNADA_680EU_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_HP_JORNADA_6XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_JORNADA_680DE
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_JORNADA_680DE_NUM	4
#define PLATID_MACH_HP_JORNADA_680DE	\
  ((PLATID_MACH_HP_JORNADA_680DE_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_HP_JORNADA_6XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_JORNADA_690
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_JORNADA_690_NUM	5
#define PLATID_MACH_HP_JORNADA_690	\
  ((PLATID_MACH_HP_JORNADA_690_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_HP_JORNADA_6XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_JORNADA_690JP
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_JORNADA_690JP_NUM	6
#define PLATID_MACH_HP_JORNADA_690JP	\
  ((PLATID_MACH_HP_JORNADA_690JP_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_HP_JORNADA_6XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_JORNADA_690EU
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_JORNADA_690EU_NUM	7
#define PLATID_MACH_HP_JORNADA_690EU	\
  ((PLATID_MACH_HP_JORNADA_690EU_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_HP_JORNADA_6XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_JORNADA_690DE
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_JORNADA_690DE_NUM	8
#define PLATID_MACH_HP_JORNADA_690DE	\
  ((PLATID_MACH_HP_JORNADA_690DE_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_HP_JORNADA_6XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_JORNADA_680FR
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_JORNADA_680FR_NUM	9
#define PLATID_MACH_HP_JORNADA_680FR	\
  ((PLATID_MACH_HP_JORNADA_680FR_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_HP_JORNADA_6XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_JORNADA_690FR
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_JORNADA_690FR_NUM	10
#define PLATID_MACH_HP_JORNADA_690FR	\
  ((PLATID_MACH_HP_JORNADA_690FR_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_HP_JORNADA_6XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_JORNADA_680SV
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_JORNADA_680SV_NUM	11
#define PLATID_MACH_HP_JORNADA_680SV	\
  ((PLATID_MACH_HP_JORNADA_680SV_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_HP_JORNADA_6XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_JORNADA_690SV
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_JORNADA_690SV_NUM	12
#define PLATID_MACH_HP_JORNADA_690SV	\
  ((PLATID_MACH_HP_JORNADA_690SV_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_HP_JORNADA_6XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_JORNADA_680ES
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_JORNADA_680ES_NUM	13
#define PLATID_MACH_HP_JORNADA_680ES	\
  ((PLATID_MACH_HP_JORNADA_680ES_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_HP_JORNADA_6XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_JORNADA_690ES
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_JORNADA_690ES_NUM	14
#define PLATID_MACH_HP_JORNADA_690ES	\
  ((PLATID_MACH_HP_JORNADA_690ES_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_HP_JORNADA_6XX)
#endif /* hpcsh */
#ifdef hpcarm
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_JORNADA_7XX
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_JORNADA_7XX_NUM	2
#define PLATID_MACH_HP_JORNADA_7XX	\
  ((PLATID_MACH_HP_JORNADA_7XX_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_HP_JORNADA)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_JORNADA_720
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_JORNADA_720_NUM	1
#define PLATID_MACH_HP_JORNADA_720	\
  ((PLATID_MACH_HP_JORNADA_720_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_HP_JORNADA_7XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_JORNADA_720JP
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_JORNADA_720JP_NUM	2
#define PLATID_MACH_HP_JORNADA_720JP	\
  ((PLATID_MACH_HP_JORNADA_720JP_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_HP_JORNADA_7XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_JORNADA_720EU
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_JORNADA_720EU_NUM	3
#define PLATID_MACH_HP_JORNADA_720EU	\
  ((PLATID_MACH_HP_JORNADA_720EU_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_HP_JORNADA_7XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_JORNADA_720DE
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_JORNADA_720DE_NUM	4
#define PLATID_MACH_HP_JORNADA_720DE	\
  ((PLATID_MACH_HP_JORNADA_720DE_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_HP_JORNADA_7XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_JORNADA_720FR
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_JORNADA_720FR_NUM	5
#define PLATID_MACH_HP_JORNADA_720FR	\
  ((PLATID_MACH_HP_JORNADA_720FR_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_HP_JORNADA_7XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_JORNADA_720SV
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_JORNADA_720SV_NUM	6
#define PLATID_MACH_HP_JORNADA_720SV	\
  ((PLATID_MACH_HP_JORNADA_720SV_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_HP_JORNADA_7XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_JORNADA_720ES
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_JORNADA_720ES_NUM	7
#define PLATID_MACH_HP_JORNADA_720ES	\
  ((PLATID_MACH_HP_JORNADA_720ES_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_HP_JORNADA_7XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_JORNADA_8XX
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_JORNADA_8XX_NUM	3
#define PLATID_MACH_HP_JORNADA_8XX	\
  ((PLATID_MACH_HP_JORNADA_8XX_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_HP_JORNADA)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_JORNADA_820
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_JORNADA_820_NUM	1
#define PLATID_MACH_HP_JORNADA_820	\
  ((PLATID_MACH_HP_JORNADA_820_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_HP_JORNADA_8XX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HP_JORNADA_820JP
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HP_JORNADA_820JP_NUM	2
#define PLATID_MACH_HP_JORNADA_820JP	\
  ((PLATID_MACH_HP_JORNADA_820JP_NUM << PLATID_SUBMODEL_SHIFT)| \
    PLATID_MACH_HP_JORNADA_8XX)
#endif /* hpcarm */
#ifdef hpcsh
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HITACHI
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HITACHI_NUM	12
#define PLATID_MACH_HITACHI	\
  ((PLATID_MACH_HITACHI_NUM << PLATID_VENDOR_SHIFT))
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HITACHI_PERSONA
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HITACHI_PERSONA_NUM	1
#define PLATID_MACH_HITACHI_PERSONA	\
  ((PLATID_MACH_HITACHI_PERSONA_NUM << PLATID_SERIES_SHIFT)| \
    PLATID_MACH_HITACHI)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HITACHI_PERSONA_HPW230JC
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HITACHI_PERSONA_HPW230JC_NUM	1
#define PLATID_MACH_HITACHI_PERSONA_HPW230JC	\
  ((PLATID_MACH_HITACHI_PERSONA_HPW230JC_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_HITACHI_PERSONA)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HITACHI_PERSONA_HPW50PAD
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HITACHI_PERSONA_HPW50PAD_NUM	2
#define PLATID_MACH_HITACHI_PERSONA_HPW50PAD	\
  ((PLATID_MACH_HITACHI_PERSONA_HPW50PAD_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_HITACHI_PERSONA)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HITACHI_PERSONA_HPW200EC
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HITACHI_PERSONA_HPW200EC_NUM	3
#define PLATID_MACH_HITACHI_PERSONA_HPW200EC	\
  ((PLATID_MACH_HITACHI_PERSONA_HPW200EC_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_HITACHI_PERSONA)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_HITACHI_PERSONA_HPW650PA
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_HITACHI_PERSONA_HPW650PA_NUM	4
#define PLATID_MACH_HITACHI_PERSONA_HPW650PA	\
  ((PLATID_MACH_HITACHI_PERSONA_HPW650PA_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_HITACHI_PERSONA)
#endif /* hpcsh */
#ifdef hpcsh
#ifndef SPEC_PLATFORM
#define SPEC_MACH_LGE
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_LGE_NUM	13
#define PLATID_MACH_LGE	\
  ((PLATID_MACH_LGE_NUM << PLATID_VENDOR_SHIFT))
#ifndef SPEC_PLATFORM
#define SPEC_MACH_LGE_PHENOM
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_LGE_PHENOM_NUM	1
#define PLATID_MACH_LGE_PHENOM	\
  ((PLATID_MACH_LGE_PHENOM_NUM << PLATID_SERIES_SHIFT)| \
    PLATID_MACH_LGE)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_LGE_PHENOM_H220C
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_LGE_PHENOM_H220C_NUM	1
#define PLATID_MACH_LGE_PHENOM_H220C	\
  ((PLATID_MACH_LGE_PHENOM_H220C_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_LGE_PHENOM)
#endif /* hpcsh */
#ifdef hpcmips
#ifndef SPEC_PLATFORM
#define SPEC_MACH_LASER5
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_LASER5_NUM	14
#define PLATID_MACH_LASER5	\
  ((PLATID_MACH_LASER5_NUM << PLATID_VENDOR_SHIFT))
#ifndef SPEC_PLATFORM
#define SPEC_MACH_LASER5_L
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_LASER5_L_NUM	1
#define PLATID_MACH_LASER5_L	\
  ((PLATID_MACH_LASER5_L_NUM << PLATID_SERIES_SHIFT)| \
    PLATID_MACH_LASER5)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_LASER5_L_CARD
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_LASER5_L_CARD_NUM	1
#define PLATID_MACH_LASER5_L_CARD	\
  ((PLATID_MACH_LASER5_L_CARD_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_LASER5_L)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_LASER5_L_BOARD
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_LASER5_L_BOARD_NUM	2
#define PLATID_MACH_LASER5_L_BOARD	\
  ((PLATID_MACH_LASER5_L_BOARD_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_LASER5_L)
#endif /* hpcmips */
#ifdef hpcmips
#ifndef SPEC_PLATFORM
#define SPEC_MACH_AGENDA
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_AGENDA_NUM	15
#define PLATID_MACH_AGENDA	\
  ((PLATID_MACH_AGENDA_NUM << PLATID_VENDOR_SHIFT))
#ifndef SPEC_PLATFORM
#define SPEC_MACH_AGENDA_VR
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_AGENDA_VR_NUM	1
#define PLATID_MACH_AGENDA_VR	\
  ((PLATID_MACH_AGENDA_VR_NUM << PLATID_SERIES_SHIFT)| \
    PLATID_MACH_AGENDA)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_AGENDA_VR_VR3
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_AGENDA_VR_VR3_NUM	1
#define PLATID_MACH_AGENDA_VR_VR3	\
  ((PLATID_MACH_AGENDA_VR_VR3_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_AGENDA_VR)
#endif /* hpcmips */
#ifdef hpcarm
#ifndef SPEC_PLATFORM
#define SPEC_MACH_PSIONTEKLOGIX
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_PSIONTEKLOGIX_NUM	16
#define PLATID_MACH_PSIONTEKLOGIX	\
  ((PLATID_MACH_PSIONTEKLOGIX_NUM << PLATID_VENDOR_SHIFT))
#ifndef SPEC_PLATFORM
#define SPEC_MACH_PSIONTEKLOGIX_NETBOOK
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_PSIONTEKLOGIX_NETBOOK_NUM	1
#define PLATID_MACH_PSIONTEKLOGIX_NETBOOK	\
  ((PLATID_MACH_PSIONTEKLOGIX_NETBOOK_NUM << PLATID_SERIES_SHIFT)| \
    PLATID_MACH_PSIONTEKLOGIX)
#ifndef SPEC_PLATFORM
#define SPEC_MACH_PSIONTEKLOGIX_NETBOOK_PRO
#endif /* !SPEC_PLATFORM */
#define PLATID_MACH_PSIONTEKLOGIX_NETBOOK_PRO_NUM	1
#define PLATID_MACH_PSIONTEKLOGIX_NETBOOK_PRO	\
  ((PLATID_MACH_PSIONTEKLOGIX_NETBOOK_PRO_NUM << PLATID_MODEL_SHIFT)| \
    PLATID_MACH_PSIONTEKLOGIX_NETBOOK)
#endif /* hpcarm */
