/*	$NetBSD: coff_machdep.h,v 1.1 2006/09/01 21:26:18 uwe Exp $	*/

#ifndef _LANDISK_COFF_MACHDEP_H_
#define _LANDISK_COFF_MACHDEP_H_

#include <sh3/coff_machdep.h>

#endif /* _LANDISK_COFF_MACHDEP_H_ */
