/*	$NetBSD: mq100.h,v 1.3 2005/12/11 12:17:29 christos Exp $	*/

#ifndef _HPCBOOT_SH_DEV_MQ100_H_
#define	_HPCBOOT_SH_DEV_MQ100_H_

#include "../../../../../hpcsh/dev/mq100videoreg.h"

#endif /* _HPCBOOT_SH_DEV_MQ100_H_ */
