/*	$NetBSD: types.h,v 1.7 2017/01/26 15:55:09 christos Exp $	*/

#ifndef _ALGOR_TYPES_H_
#define	_ALGOR_TYPES_H_

#include <mips/types.h>

#define	__HAVE_NEW_STYLE_BUS_H
#define	__HAVE_COMPAT_NETBSD32

#endif
