/*	$NetBSD: hpcmenu.rc,v 1.21 2011/11/23 15:49:58 nonaka Exp $	*/

/*-
 * Copyright (c) 2001 The NetBSD Foundation, Inc.
 * All rights reserved.
 *
 * This code is derived from software contributed to The NetBSD Foundation
 * by UCHIYAMA Yasushi.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE NETBSD FOUNDATION, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#define APSTUDIO_READONLY_SYMBOLS
#define APSTUDIO_HIDDEN_SYMBOLS
#include <windows.h>
#undef APSTUDIO_HIDDEN_SYMBOLS
#include "resource.h"
#undef APSTUDIO_READONLY_SYMBOLS

LANGUAGE LANG_NEUTRAL, SUBLANG_NEUTRAL

STRINGTABLE DISCARDABLE
{
	IDS_APP_TITLE			"hpcboot"
	IDS_HPCMENU			"HPCBOOT"
	IDS_DIR_FFS_ROOT1		"/"
	IDS_DIR_FFS_ROOT2		"2:/"
	IDS_DIR_FFS_SMC_ROOT1		"SMC1:/"
	IDS_DIR_FAT_ROOT		"\\"
	IDS_DIR_FAT_MY_DOCUMENTS	"\\My Documents\\"
	IDS_DIR_CARD_EN			"\\Storage Card\\"
	IDS_DIR_CARD_EN1		"\\Storage Card1\\"
	IDS_DIR_CARD_EN2		"\\Storage Card2\\"
	IDS_DIR_CARD_JP			L"\\\xFF92\xFF93\xFF98 \xFF76\xFF70\xFF84\xFF9E\\"
	IDS_DIR_CARD_JP1		L"\\\xFF92\xFF93\xFF98 \xFF76\xFF70\xFF84\xFF9E1\\"
	IDS_DIR_CARD_JP2		L"\\\xFF92\xFF93\xFF98 \xFF76\xFF70\xFF84\xFF9E2\\"
	IDS_DIR_CARD_MMCARD		"\\Multimedia Card\\"
	IDS_DIR_CARD_SDMMCARD		"\\SD-MMC Card\\"
	IDS_DIR_CARD_MINISD_JP		L"\\miniSD\xFF76\xFF70\xFF84\xFF9E\\"
	IDS_DIR_CARD_MICROSD_JP		L"\\microSD\x30AB\x30FC\x30C9\\"
	IDS_DIR_USER_DEFINED		"user defined"
}

IDI_ICON		ICON			"hpcboot.ico"
IDI_HPCMENU_MAIN	BITMAP	DISCARDABLE	"kernel.bmp"
IDI_HPCMENU_OPTION	BITMAP	DISCARDABLE	"option.bmp"
IDI_HPCMENU_CONSOLE	BITMAP	DISCARDABLE	"console.bmp"

WMain DIALOG DISCARDABLE 0, 0, 0, 0
STYLE WS_CHILD
{
	// kernel directory
	COMBOBOX	IDC_MAIN_DIR,			5,  5, 70, 80,
		WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL | WS_TABSTOP |
		CBS_DROPDOWN | CBS_AUTOHSCROLL
	// kernel file name
	EDITTEXT	IDC_MAIN_KERNEL			75, 5, 40, 12,
		WS_CHILD | WS_VISIBLE | WS_BORDER | WS_TABSTOP |
		ES_LEFT | ES_AUTOHSCROLL
	// platform select
	COMBOBOX	IDC_MAIN_PLATFORM,		5, 20, 110, 64,
		WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL | WS_TABSTOP |
		CBS_DROPDOWN | CBS_AUTOHSCROLL
	// root file system select
	AUTORADIOBUTTON	"wd",	IDC_MAIN_ROOT_WD,	10, 40, 22, 10
	AUTORADIOBUTTON	"sd",	IDC_MAIN_ROOT_SD,	35, 40, 22, 10
	AUTORADIOBUTTON	"md",	IDC_MAIN_ROOT_MD,	60, 40, 22, 10
	AUTORADIOBUTTON	"nfs",	IDC_MAIN_ROOT_NFS,	85, 40, 22, 10
	AUTORADIOBUTTON	"dk",	IDC_MAIN_ROOT_DK,	10, 51, 22, 10
	AUTORADIOBUTTON	"ld",	IDC_MAIN_ROOT_LD,	35, 51, 22, 10
	EDITTEXT	IDC_MAIN_ROOT_MD_OPS		10, 64, 100, 12,
		WS_CHILD | WS_VISIBLE | WS_BORDER | WS_TABSTOP |
		ES_LEFT | ES_AUTOHSCROLL
	GROUPBOX	"root file system"	IDC_MAIN_ROOT_OPTION
							5, 31, 110, 48
	// boot options
	AUTOCHECKBOX	"boot verbosely"	IDC_MAIN_OPTION_V
							5, 59, 120, 9
	AUTOCHECKBOX	"boot to single user"	IDC_MAIN_OPTION_S
							5, 68, 120, 9
	AUTOCHECKBOX	"ask for file name to boot from" IDC_MAIN_OPTION_A
							5, 77, 120, 9
	AUTOCHECKBOX	"break into kernel debugger" IDC_MAIN_OPTION_D
							5, 86, 120, 9
	AUTOCHECKBOX	"serial console",	IDC_MAIN_OPTION_H,
							5, 95, 55, 9
	COMBOBOX	IDC_MAIN_OPTION_H_SPEED		65, 93, 50, 40,
		WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL | WS_TABSTOP |
		CBS_DROPDOWN | CBS_AUTOHSCROLL
}

WOption DIALOG DISCARDABLE 0, 0, 0, 0
STYLE WS_CHILD
{
	AUTOCHECKBOX	"Auto Boot",		IDC_OPT_AUTO,	5, 0,  50, 12
	EDITTEXT	IDC_OPT_AUTO_INPUT, 55, 0, 24, 15,
		WS_CHILD | WS_VISIBLE | WS_BORDER | ES_LEFT
	AUTOCHECKBOX	"Reverse Video",	IDC_OPT_VIDEO,	5, 14, 80, 12
	AUTOCHECKBOX	"Pause Before Boot",	IDC_OPT_PAUSE,	5, 26, 80, 12
	AUTOCHECKBOX	"Load Debug Info",	IDC_OPT_DEBUG,	5, 38, 80, 12
	AUTOCHECKBOX	"Safety Message",	IDC_OPT_SAFETY,	5, 50, 80, 12
	LTEXT		"Extra Kernel Options:"		-1, 5, 62, 80, 12
	EDITTEXT	IDC_OPT_EXTKOPT		5, 74, 120, 12,
		WS_CHILD | WS_VISIBLE | WS_BORDER | WS_TABSTOP |
		ES_LEFT | ES_AUTOHSCROLL
}

WConsole DIALOG DISCARDABLE 0, 0, 0, 0
STYLE WS_CHILD
{
	AUTOCHECKBOX	"", IDC_CONS_CHK7,  5, 0, 10, 10
	AUTOCHECKBOX	"", IDC_CONS_CHK6, 15, 0, 10, 10
	AUTOCHECKBOX	"", IDC_CONS_CHK5, 25, 0, 10, 10
	AUTOCHECKBOX	"", IDC_CONS_CHK4, 35, 0, 10, 10
	AUTOCHECKBOX	"", IDC_CONS_CHK3, 45, 0, 10, 10
	AUTOCHECKBOX	"", IDC_CONS_CHK2, 55, 0, 10, 10
	AUTOCHECKBOX	"", IDC_CONS_CHK1, 65, 0, 10, 10
	AUTOCHECKBOX	"", IDC_CONS_CHK0, 75, 0, 10, 10
	PUSHBUTTON	"a", IDC_CONS_BTN0, 85, 0, 10, 10,
		WS_CHILD | WS_VISIBLE
	PUSHBUTTON	"b", IDC_CONS_BTN1, 95, 0, 10, 10,
		WS_CHILD | WS_VISIBLE
	PUSHBUTTON	"c", IDC_CONS_BTN2, 105, 0, 10, 10,
		WS_CHILD | WS_VISIBLE
	PUSHBUTTON	"d", IDC_CONS_BTN3, 115, 0, 10, 10,
		WS_CHILD | WS_VISIBLE
	AUTOCHECKBOX	"save to file", IDC_CONS_FILESAVE,  5, 10, 120, 10
	EDITTEXT	IDC_CONS_FILENAME   5, 20, 120, 12,
		WS_CHILD | WS_VISIBLE | WS_BORDER | WS_TABSTOP |
		ES_LEFT | ES_AUTOHSCROLL
	EDITTEXT	IDC_CONS_EDIT, 0, 0, 0, 0,
		WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL |
		ES_AUTOHSCROLL | ES_AUTOVSCROLL | ES_LEFT | ES_MULTILINE |
		ES_WANTRETURN
}
