/*	$NetBSD: types.h,v 1.5 2021/01/23 19:38:52 christos Exp $	*/
#ifndef _DREAMCAST_TYPES_H_
#define	_DREAMCAST_TYPES_H_

#include <sh3/types.h>
#define	__HAVE_BUS_SPACE_8

#endif /* _DREAMCAST_TYPES_H_ */
