/*	$NetBSD: elf_machdep.h,v 1.1 2000/12/11 18:19:14 marcus Exp $	*/
#ifndef _DREAMCAST_ELF_MACHDEP_H_
#define _DREAMCAST_ELF_MACHDEP_H_

#include <sh3/elf_machdep.h>

#endif /* _DREAMCAST_ELF_MACHDEP_H_ */
