/*	$NetBSD: cpufunc.h,v 1.1.1.1 1995/03/26 07:12:05 leo Exp $	*/

/*
 * Functions to provide access to special cpu instructions.
 */
#ifndef _MACHINE_CPUFUNC_H_
#define _MACHINE_CPUFUNC_H_
#endif  /* !_MACHINE_CPUFUNC_H_ */
