/*	$NetBSD: spkr.h,v 1.1 2000/02/29 15:21:31 nonaka Exp $	*/

/*
 * spkr.h -- interface definitions for speaker ioctl()
 */

#ifndef _PREP_SPKR_H_
#define _PREP_SPKR_H_

#include <sys/ioctl.h>

#include <dev/isa/spkrio.h>

#endif /* _PREP_SPKR_H_ */
