/* $NetBSD: bus_defs.h,v 1.2 2011/07/09 17:22:08 dyoung Exp $ */

#ifndef _ALGOR_BUS_DEFS_H_
#define	_ALGOR_BUS_DEFS_H_
#define _MIPS_NEED_BUS_DMA_BOUNCE

#include <mips/bus_space_defs.h>
#include <mips/bus_dma_defs.h>

#endif /* _ALGOR_BUS_DEFS_H_ */
