/* $NetBSD: clk_dp8570a.c,v 1.1 2001/05/14 18:22:59 drochner Exp $ */

/*
 * tbd. For now we boot from a running OS-9 system, so
 * the clock is already initialized.
 * The interrupt vector is hardwired.
 */
